# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._createable_api_resource import CreateableAPIResource
from stripe._expandable_field import ExpandableField
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe._test_helpers import APIResourceTestHelpers
from stripe._updateable_api_resource import UpdateableAPIResource
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, List, Optional, cast, overload
from typing_extensions import Literal, Type, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._balance_transaction import BalanceTransaction
    from stripe._file import File
    from stripe.issuing._transaction import Transaction
    from stripe.params.issuing._dispute_close_params import DisputeCloseParams
    from stripe.params.issuing._dispute_create_params import (
        DisputeCreateParams,
    )
    from stripe.params.issuing._dispute_list_params import DisputeListParams
    from stripe.params.issuing._dispute_modify_params import (
        DisputeModifyParams,
    )
    from stripe.params.issuing._dispute_provisional_credit_params import (
        DisputeProvisionalCreditParams,
    )
    from stripe.params.issuing._dispute_retrieve_params import (
        DisputeRetrieveParams,
    )
    from stripe.params.issuing._dispute_simulate_network_lifecycle_dispute_response_params import (
        DisputeSimulateNetworkLifecycleDisputeResponseParams,
    )
    from stripe.params.issuing._dispute_simulate_network_lifecycle_pre_arbitration_response_params import (
        DisputeSimulateNetworkLifecyclePreArbitrationResponseParams,
    )
    from stripe.params.issuing._dispute_simulate_network_lifecycle_pre_arbitration_submission_params import (
        DisputeSimulateNetworkLifecyclePreArbitrationSubmissionParams,
    )
    from stripe.params.issuing._dispute_submit_params import (
        DisputeSubmitParams,
    )


class Dispute(
    CreateableAPIResource["Dispute"],
    ListableAPIResource["Dispute"],
    UpdateableAPIResource["Dispute"],
):
    """
    As a [card issuer](https://docs.stripe.com/issuing), you can dispute transactions that the cardholder does not recognize, suspects to be fraudulent, or has other issues with.

    Related guide: [Issuing disputes](https://docs.stripe.com/issuing/purchases/disputes)
    """

    OBJECT_NAME: ClassVar[Literal["issuing.dispute"]] = "issuing.dispute"

    class CryptoTransaction(StripeObject):
        class CryptoTransactionConfirmed(StripeObject):
            class Fee(StripeObject):
                amount: str
                """
                The fee amount.
                """
                currency: str
                """
                The fee currency.
                """
                type: str
                """
                The fee type.
                """

            amount: str
            """
            The crypto amount for the confirmed transaction.
            """
            amount_mcc_upcharged: Optional[str]
            """
            The upcharged MCC amount, if one was applied.
            """
            chain: str
            """
            The blockchain network for the confirmed transaction.
            """
            confirmed_at: int
            """
            When the transaction was confirmed onchain.
            """
            currency: str
            """
            The currency of the crypto transaction amount.
            """
            fees: List[Fee]
            """
            Fees associated with the transaction.
            """
            from_address: str
            """
            The source wallet address for the transaction.
            """
            memo: Optional[str]
            """
            Memo metadata attached to the transaction, if present.
            """
            to_address: str
            """
            The destination wallet address for the transaction.
            """
            transaction_hash: str
            """
            The blockchain transaction hash.
            """
            _inner_class_types = {"fees": Fee}

        class CryptoTransactionFailed(StripeObject):
            class Fee(StripeObject):
                amount: str
                """
                The fee amount.
                """
                currency: str
                """
                The fee currency.
                """
                type: str
                """
                The fee type.
                """

            amount: str
            """
            The crypto amount for the failed transaction.
            """
            amount_mcc_upcharged: Optional[str]
            """
            The upcharged MCC amount, if one was applied.
            """
            chain: str
            """
            The blockchain network for the failed transaction.
            """
            currency: str
            """
            The currency of the crypto transaction amount.
            """
            failed_at: int
            """
            When the transaction failed.
            """
            failure_reason: str
            """
            The reason the transaction failed.
            """
            fees: List[Fee]
            """
            Fees associated with the transaction.
            """
            from_address: str
            """
            The source wallet address for the attempted transaction.
            """
            memo: Optional[str]
            """
            Memo metadata attached to the transaction, if present.
            """
            to_address: Optional[str]
            """
            The destination wallet address for the attempted transaction when one exists.
            """
            transaction_hash: Optional[str]
            """
            The blockchain transaction hash when one exists.
            """
            _inner_class_types = {"fees": Fee}

        crypto_transaction_confirmed: Optional[CryptoTransactionConfirmed]
        """
        The confirmed crypto transaction details when `type` is `crypto_transaction_confirmed`; otherwise null.
        """
        crypto_transaction_failed: Optional[CryptoTransactionFailed]
        """
        The failed crypto transaction details when `type` is `crypto_transaction_failed`; otherwise null.
        """
        type: str
        """
        The crypto transaction variant for this array entry.
        """
        _inner_class_types = {
            "crypto_transaction_confirmed": CryptoTransactionConfirmed,
            "crypto_transaction_failed": CryptoTransactionFailed,
        }

    class Evidence(StripeObject):
        class Canceled(StripeObject):
            additional_documentation: Optional[ExpandableField["File"]]
            """
            (ID of a [file upload](https://stripe.com/docs/guides/file-upload)) Additional documentation supporting the dispute.
            """
            canceled_at: Optional[int]
            """
            Date when order was canceled.
            """
            cancellation_policy_provided: Optional[bool]
            """
            Whether the cardholder was provided with a cancellation policy.
            """
            cancellation_reason: Optional[str]
            """
            Reason for canceling the order.
            """
            expected_at: Optional[int]
            """
            Date when the cardholder expected to receive the product.
            """
            explanation: Optional[str]
            """
            Explanation of why the cardholder is disputing this transaction.
            """
            product_description: Optional[str]
            """
            Description of the merchandise or service that was purchased.
            """
            product_type: Optional[Literal["merchandise", "service"]]
            """
            Whether the product was a merchandise or service.
            """
            return_status: Optional[Literal["merchant_rejected", "successful"]]
            """
            Result of cardholder's attempt to return the product.
            """
            returned_at: Optional[int]
            """
            Date when the product was returned or attempted to be returned.
            """

        class Duplicate(StripeObject):
            additional_documentation: Optional[ExpandableField["File"]]
            """
            (ID of a [file upload](https://stripe.com/docs/guides/file-upload)) Additional documentation supporting the dispute.
            """
            card_statement: Optional[ExpandableField["File"]]
            """
            (ID of a [file upload](https://stripe.com/docs/guides/file-upload)) Copy of the card statement showing that the product had already been paid for.
            """
            cash_receipt: Optional[ExpandableField["File"]]
            """
            (ID of a [file upload](https://stripe.com/docs/guides/file-upload)) Copy of the receipt showing that the product had been paid for in cash.
            """
            check_image: Optional[ExpandableField["File"]]
            """
            (ID of a [file upload](https://stripe.com/docs/guides/file-upload)) Image of the front and back of the check that was used to pay for the product.
            """
            explanation: Optional[str]
            """
            Explanation of why the cardholder is disputing this transaction.
            """
            original_transaction: Optional[str]
            """
            Transaction (e.g., ipi_...) that the disputed transaction is a duplicate of. Of the two or more transactions that are copies of each other, this is original undisputed one.
            """

        class Fraudulent(StripeObject):
            additional_documentation: Optional[ExpandableField["File"]]
            """
            (ID of a [file upload](https://stripe.com/docs/guides/file-upload)) Additional documentation supporting the dispute.
            """
            explanation: Optional[str]
            """
            Explanation of why the cardholder is disputing this transaction.
            """

        class MerchandiseNotAsDescribed(StripeObject):
            additional_documentation: Optional[ExpandableField["File"]]
            """
            (ID of a [file upload](https://stripe.com/docs/guides/file-upload)) Additional documentation supporting the dispute.
            """
            explanation: Optional[str]
            """
            Explanation of why the cardholder is disputing this transaction.
            """
            received_at: Optional[int]
            """
            Date when the product was received.
            """
            return_description: Optional[str]
            """
            Description of the cardholder's attempt to return the product.
            """
            return_status: Optional[Literal["merchant_rejected", "successful"]]
            """
            Result of cardholder's attempt to return the product.
            """
            returned_at: Optional[int]
            """
            Date when the product was returned or attempted to be returned.
            """

        class NoValidAuthorization(StripeObject):
            additional_documentation: Optional[ExpandableField["File"]]
            """
            (ID of a [file upload](https://stripe.com/docs/guides/file-upload)) Additional documentation supporting the dispute.
            """
            explanation: Optional[str]
            """
            Explanation of why the cardholder is disputing this transaction.
            """

        class NotReceived(StripeObject):
            additional_documentation: Optional[ExpandableField["File"]]
            """
            (ID of a [file upload](https://stripe.com/docs/guides/file-upload)) Additional documentation supporting the dispute.
            """
            expected_at: Optional[int]
            """
            Date when the cardholder expected to receive the product.
            """
            explanation: Optional[str]
            """
            Explanation of why the cardholder is disputing this transaction.
            """
            product_description: Optional[str]
            """
            Description of the merchandise or service that was purchased.
            """
            product_type: Optional[Literal["merchandise", "service"]]
            """
            Whether the product was a merchandise or service.
            """

        class Other(StripeObject):
            additional_documentation: Optional[ExpandableField["File"]]
            """
            (ID of a [file upload](https://stripe.com/docs/guides/file-upload)) Additional documentation supporting the dispute.
            """
            explanation: Optional[str]
            """
            Explanation of why the cardholder is disputing this transaction.
            """
            product_description: Optional[str]
            """
            Description of the merchandise or service that was purchased.
            """
            product_type: Optional[Literal["merchandise", "service"]]
            """
            Whether the product was a merchandise or service.
            """

        class ServiceNotAsDescribed(StripeObject):
            additional_documentation: Optional[ExpandableField["File"]]
            """
            (ID of a [file upload](https://stripe.com/docs/guides/file-upload)) Additional documentation supporting the dispute.
            """
            canceled_at: Optional[int]
            """
            Date when order was canceled.
            """
            cancellation_reason: Optional[str]
            """
            Reason for canceling the order.
            """
            explanation: Optional[str]
            """
            Explanation of why the cardholder is disputing this transaction.
            """
            received_at: Optional[int]
            """
            Date when the product was received.
            """

        canceled: Optional[Canceled]
        duplicate: Optional[Duplicate]
        fraudulent: Optional[Fraudulent]
        merchandise_not_as_described: Optional[MerchandiseNotAsDescribed]
        no_valid_authorization: Optional[NoValidAuthorization]
        not_received: Optional[NotReceived]
        other: Optional[Other]
        reason: Literal[
            "canceled",
            "duplicate",
            "fraudulent",
            "merchandise_not_as_described",
            "no_valid_authorization",
            "not_received",
            "other",
            "service_not_as_described",
        ]
        """
        The reason for filing the dispute. Its value will match the field containing the evidence.
        """
        service_not_as_described: Optional[ServiceNotAsDescribed]
        _inner_class_types = {
            "canceled": Canceled,
            "duplicate": Duplicate,
            "fraudulent": Fraudulent,
            "merchandise_not_as_described": MerchandiseNotAsDescribed,
            "no_valid_authorization": NoValidAuthorization,
            "not_received": NotReceived,
            "other": Other,
            "service_not_as_described": ServiceNotAsDescribed,
        }

    class NetworkLifecycle(StripeObject):
        class DisputeResponse(StripeObject):
            error: Optional[str]
            """
            Error message if processing the acquiring merchant's initial dispute response failed.
            """
            merchant_evidence_files: Optional[List[str]]
            """
            Array of [File](https://docs.stripe.com/api/files) ids containing evidence the acquiring merchant provided in support of their initial dispute response.
            """

        class PreArbitrationResponse(StripeObject):
            error: Optional[str]
            """
            Error message if processing the acquiring merchant's pre-arbitration response failed.
            """
            merchant_evidence_files: Optional[List[str]]
            """
            Array of [File](https://docs.stripe.com/api/files) ids containing evidence the acquiring merchant provided with their pre-arbitration response.
            """

        class PreArbitrationSubmission(StripeObject):
            error: Optional[str]
            """
            Error message if processing the acquiring merchant's pre-arbitration submission failed.
            """
            merchant_evidence_files: Optional[List[str]]
            """
            Array of [File](https://docs.stripe.com/api/files) ids containing evidence the acquiring merchant provided with their pre-arbitration submission.
            """

        dispute_response: Optional[DisputeResponse]
        """
        Information related to the acquiring merchant's initial response to this dispute.
        """
        pre_arbitration_response: Optional[PreArbitrationResponse]
        """
        Information related to the acquiring merchant's pre-arbitration response for this dispute.
        """
        pre_arbitration_submission: Optional[PreArbitrationSubmission]
        """
        Information related to the acquiring merchant's pre-arbitration submission for this dispute.
        """
        _inner_class_types = {
            "dispute_response": DisputeResponse,
            "pre_arbitration_response": PreArbitrationResponse,
            "pre_arbitration_submission": PreArbitrationSubmission,
        }

    class ProvisionalCredit(StripeObject):
        grant_deadline: Optional[int]
        """
        The time by which the platform must grant a provisional credit to the consumer.
        """
        granted_at: Optional[int]
        """
        The time at which the platform reported granting the provisional credit.
        """
        revocable_after: Optional[int]
        """
        The earliest time after which the platform can revoke the provisional credit.
        """
        revoked_at: Optional[int]
        """
        The time at which the platform reported revoking the provisional credit.
        """
        status: Literal[
            "delinquent",
            "granted",
            "not_required",
            "permanent",
            "required",
            "revocable",
            "revocation_notice_period",
            "revoked",
        ]
        """
        The status of the provisional credit obligation.
        """

    class Redaction(StripeObject):
        status: Literal["processing", "redacted", "validated"]
        """
        Indicates whether this object and its related objects have been redacted or not.
        """

    class Treasury(StripeObject):
        debit_reversal: Optional[str]
        """
        The Treasury [DebitReversal](https://docs.stripe.com/api/treasury/debit_reversals) representing this Issuing dispute
        """
        received_debit: str
        """
        The Treasury [ReceivedDebit](https://docs.stripe.com/api/treasury/received_debits) that is being disputed.
        """

    amount: int
    """
    Disputed amount in the card's currency and in the [smallest currency unit](https://docs.stripe.com/currencies#zero-decimal). Usually the amount of the `transaction`, but can differ (usually because of currency fluctuation).
    """
    balance_transactions: Optional[List["BalanceTransaction"]]
    """
    List of balance transactions associated with the dispute.
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    crypto_transactions: Optional[List[CryptoTransaction]]
    """
    Array of onchain crypto transactions linked to this resource.
    """
    currency: str
    """
    The currency the `transaction` was made in.
    """
    evidence: Evidence
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    loss_reason: Optional[
        Literal[
            "cardholder_authentication_issuer_liability",
            "eci5_token_transaction_with_tavv",
            "excess_disputes_in_timeframe",
            "has_not_met_the_minimum_dispute_amount_requirements",
            "invalid_duplicate_dispute",
            "invalid_incorrect_amount_dispute",
            "invalid_no_authorization",
            "invalid_use_of_disputes",
            "merchandise_delivered_or_shipped",
            "merchandise_or_service_as_described",
            "not_cancelled",
            "other",
            "refund_issued",
            "submitted_beyond_allowable_time_limit",
            "transaction_3ds_required",
            "transaction_approved_after_prior_fraud_dispute",
            "transaction_authorized",
            "transaction_electronically_read",
            "transaction_qualifies_for_visa_easy_payment_service",
            "transaction_unattended",
        ]
    ]
    """
    The enum that describes the dispute loss outcome. If the dispute is not lost, this field will be absent. New enum values may be added in the future, so be sure to handle unknown values.
    """
    metadata: UntypedStripeObject[str]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    network_lifecycle: Optional[NetworkLifecycle]
    """
    Incoming information from the card network for this dispute. Includes the acquiring merchant's initial response, pre-arbitration submission, and pre-arbitration response to the dispute.
    """
    object: Literal["issuing.dispute"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    provisional_credit: Optional[ProvisionalCredit]
    """
    Provisional credit details for this dispute.
    """
    redaction: Optional[Redaction]
    """
    Redaction status of this dispute. If the dispute is not redacted, this field will be null.
    """
    status: Literal["expired", "lost", "submitted", "unsubmitted", "won"]
    """
    Current status of the dispute.
    """
    transaction: ExpandableField["Transaction"]
    """
    The transaction being disputed.
    """
    treasury: Optional[Treasury]
    """
    [Treasury](https://docs.stripe.com/api/treasury) details related to this dispute if it was created on a [FinancialAccount](https://docs.stripe.com/api/treasury/financial_accounts)
    """

    @classmethod
    def create(cls, **params: Unpack["DisputeCreateParams"]) -> "Dispute":
        """
        Creates an Issuing Dispute object. Individual pieces of evidence within the evidence object are optional at this point. Stripe only validates that required evidence is present during submission. Refer to [Dispute reasons and evidence](https://docs.stripe.com/docs/issuing/purchases/disputes#dispute-reasons-and-evidence) for more details about evidence requirements.
        """
        return cast(
            "Dispute",
            cls._static_request(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    async def create_async(
        cls, **params: Unpack["DisputeCreateParams"]
    ) -> "Dispute":
        """
        Creates an Issuing Dispute object. Individual pieces of evidence within the evidence object are optional at this point. Stripe only validates that required evidence is present during submission. Refer to [Dispute reasons and evidence](https://docs.stripe.com/docs/issuing/purchases/disputes#dispute-reasons-and-evidence) for more details about evidence requirements.
        """
        return cast(
            "Dispute",
            await cls._static_request_async(
                "post",
                cls.class_url(),
                params=params,
            ),
        )

    @classmethod
    def list(
        cls, **params: Unpack["DisputeListParams"]
    ) -> ListObject["Dispute"]:
        """
        Returns a list of Issuing Dispute objects. The objects are sorted in descending order by creation date, with the most recently created object appearing first.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["DisputeListParams"]
    ) -> ListObject["Dispute"]:
        """
        Returns a list of Issuing Dispute objects. The objects are sorted in descending order by creation date, with the most recently created object appearing first.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def modify(
        cls, id: str, **params: Unpack["DisputeModifyParams"]
    ) -> "Dispute":
        """
        Updates the specified Issuing Dispute object by setting the values of the parameters passed. Any parameters not provided will be left unchanged. Properties on the evidence object can be unset by passing in an empty string.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "Dispute",
            cls._static_request(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    async def modify_async(
        cls, id: str, **params: Unpack["DisputeModifyParams"]
    ) -> "Dispute":
        """
        Updates the specified Issuing Dispute object by setting the values of the parameters passed. Any parameters not provided will be left unchanged. Properties on the evidence object can be unset by passing in an empty string.
        """
        url = "%s/%s" % (cls.class_url(), sanitize_id(id))
        return cast(
            "Dispute",
            await cls._static_request_async(
                "post",
                url,
                params=params,
            ),
        )

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["DisputeRetrieveParams"]
    ) -> "Dispute":
        """
        Retrieves an Issuing Dispute object.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["DisputeRetrieveParams"]
    ) -> "Dispute":
        """
        Retrieves an Issuing Dispute object.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    @classmethod
    def _cls_submit(
        cls, dispute: str, **params: Unpack["DisputeSubmitParams"]
    ) -> "Dispute":
        """
        Submits an Issuing Dispute to the card network. Stripe validates that all evidence fields required for the dispute's reason are present. For more details, see [Dispute reasons and evidence](https://docs.stripe.com/docs/issuing/purchases/disputes#dispute-reasons-and-evidence).
        """
        return cast(
            "Dispute",
            cls._static_request(
                "post",
                "/v1/issuing/disputes/{dispute}/submit".format(
                    dispute=sanitize_id(dispute)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def submit(
        dispute: str, **params: Unpack["DisputeSubmitParams"]
    ) -> "Dispute":
        """
        Submits an Issuing Dispute to the card network. Stripe validates that all evidence fields required for the dispute's reason are present. For more details, see [Dispute reasons and evidence](https://docs.stripe.com/docs/issuing/purchases/disputes#dispute-reasons-and-evidence).
        """
        ...

    @overload
    def submit(self, **params: Unpack["DisputeSubmitParams"]) -> "Dispute":
        """
        Submits an Issuing Dispute to the card network. Stripe validates that all evidence fields required for the dispute's reason are present. For more details, see [Dispute reasons and evidence](https://docs.stripe.com/docs/issuing/purchases/disputes#dispute-reasons-and-evidence).
        """
        ...

    @class_method_variant("_cls_submit")
    def submit(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["DisputeSubmitParams"]
    ) -> "Dispute":
        """
        Submits an Issuing Dispute to the card network. Stripe validates that all evidence fields required for the dispute's reason are present. For more details, see [Dispute reasons and evidence](https://docs.stripe.com/docs/issuing/purchases/disputes#dispute-reasons-and-evidence).
        """
        return cast(
            "Dispute",
            self._request(
                "post",
                "/v1/issuing/disputes/{dispute}/submit".format(
                    dispute=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_submit_async(
        cls, dispute: str, **params: Unpack["DisputeSubmitParams"]
    ) -> "Dispute":
        """
        Submits an Issuing Dispute to the card network. Stripe validates that all evidence fields required for the dispute's reason are present. For more details, see [Dispute reasons and evidence](https://docs.stripe.com/docs/issuing/purchases/disputes#dispute-reasons-and-evidence).
        """
        return cast(
            "Dispute",
            await cls._static_request_async(
                "post",
                "/v1/issuing/disputes/{dispute}/submit".format(
                    dispute=sanitize_id(dispute)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def submit_async(
        dispute: str, **params: Unpack["DisputeSubmitParams"]
    ) -> "Dispute":
        """
        Submits an Issuing Dispute to the card network. Stripe validates that all evidence fields required for the dispute's reason are present. For more details, see [Dispute reasons and evidence](https://docs.stripe.com/docs/issuing/purchases/disputes#dispute-reasons-and-evidence).
        """
        ...

    @overload
    async def submit_async(
        self, **params: Unpack["DisputeSubmitParams"]
    ) -> "Dispute":
        """
        Submits an Issuing Dispute to the card network. Stripe validates that all evidence fields required for the dispute's reason are present. For more details, see [Dispute reasons and evidence](https://docs.stripe.com/docs/issuing/purchases/disputes#dispute-reasons-and-evidence).
        """
        ...

    @class_method_variant("_cls_submit_async")
    async def submit_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["DisputeSubmitParams"]
    ) -> "Dispute":
        """
        Submits an Issuing Dispute to the card network. Stripe validates that all evidence fields required for the dispute's reason are present. For more details, see [Dispute reasons and evidence](https://docs.stripe.com/docs/issuing/purchases/disputes#dispute-reasons-and-evidence).
        """
        return cast(
            "Dispute",
            await self._request_async(
                "post",
                "/v1/issuing/disputes/{dispute}/submit".format(
                    dispute=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    class TestHelpers(APIResourceTestHelpers["Dispute"]):
        _resource_cls: Type["Dispute"]

        @classmethod
        def _cls_close(
            cls, dispute: str, **params: Unpack["DisputeCloseParams"]
        ) -> "Dispute":
            """
            Test helper: closes a test-mode Issuing dispute as won or lost.
            """
            return cast(
                "Dispute",
                cls._static_request(
                    "post",
                    "/v1/test_helpers/issuing/disputes/{dispute}/close".format(
                        dispute=sanitize_id(dispute)
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        def close(
            dispute: str, **params: Unpack["DisputeCloseParams"]
        ) -> "Dispute":
            """
            Test helper: closes a test-mode Issuing dispute as won or lost.
            """
            ...

        @overload
        def close(self, **params: Unpack["DisputeCloseParams"]) -> "Dispute":
            """
            Test helper: closes a test-mode Issuing dispute as won or lost.
            """
            ...

        @class_method_variant("_cls_close")
        def close(  # pyright: ignore[reportGeneralTypeIssues]
            self, **params: Unpack["DisputeCloseParams"]
        ) -> "Dispute":
            """
            Test helper: closes a test-mode Issuing dispute as won or lost.
            """
            return cast(
                "Dispute",
                self.resource._request(
                    "post",
                    "/v1/test_helpers/issuing/disputes/{dispute}/close".format(
                        dispute=sanitize_id(self.resource._data.get("id"))
                    ),
                    params=params,
                ),
            )

        @classmethod
        async def _cls_close_async(
            cls, dispute: str, **params: Unpack["DisputeCloseParams"]
        ) -> "Dispute":
            """
            Test helper: closes a test-mode Issuing dispute as won or lost.
            """
            return cast(
                "Dispute",
                await cls._static_request_async(
                    "post",
                    "/v1/test_helpers/issuing/disputes/{dispute}/close".format(
                        dispute=sanitize_id(dispute)
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        async def close_async(
            dispute: str, **params: Unpack["DisputeCloseParams"]
        ) -> "Dispute":
            """
            Test helper: closes a test-mode Issuing dispute as won or lost.
            """
            ...

        @overload
        async def close_async(
            self, **params: Unpack["DisputeCloseParams"]
        ) -> "Dispute":
            """
            Test helper: closes a test-mode Issuing dispute as won or lost.
            """
            ...

        @class_method_variant("_cls_close_async")
        async def close_async(  # pyright: ignore[reportGeneralTypeIssues]
            self, **params: Unpack["DisputeCloseParams"]
        ) -> "Dispute":
            """
            Test helper: closes a test-mode Issuing dispute as won or lost.
            """
            return cast(
                "Dispute",
                await self.resource._request_async(
                    "post",
                    "/v1/test_helpers/issuing/disputes/{dispute}/close".format(
                        dispute=sanitize_id(self.resource._data.get("id"))
                    ),
                    params=params,
                ),
            )

        @classmethod
        def _cls_provisional_credit(
            cls,
            dispute: str,
            **params: Unpack["DisputeProvisionalCreditParams"],
        ) -> "Dispute":
            """
            Test helper: overrides the grant_deadline and revocable_after timestamps on a test-mode Issuing dispute's provisional credit, allowing tests to simulate timer-driven status transitions without waiting for real regulatory deadlines to pass.
            """
            return cast(
                "Dispute",
                cls._static_request(
                    "post",
                    "/v1/test_helpers/issuing/disputes/{dispute}/provisional_credit".format(
                        dispute=sanitize_id(dispute)
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        def provisional_credit(
            dispute: str, **params: Unpack["DisputeProvisionalCreditParams"]
        ) -> "Dispute":
            """
            Test helper: overrides the grant_deadline and revocable_after timestamps on a test-mode Issuing dispute's provisional credit, allowing tests to simulate timer-driven status transitions without waiting for real regulatory deadlines to pass.
            """
            ...

        @overload
        def provisional_credit(
            self, **params: Unpack["DisputeProvisionalCreditParams"]
        ) -> "Dispute":
            """
            Test helper: overrides the grant_deadline and revocable_after timestamps on a test-mode Issuing dispute's provisional credit, allowing tests to simulate timer-driven status transitions without waiting for real regulatory deadlines to pass.
            """
            ...

        @class_method_variant("_cls_provisional_credit")
        def provisional_credit(  # pyright: ignore[reportGeneralTypeIssues]
            self, **params: Unpack["DisputeProvisionalCreditParams"]
        ) -> "Dispute":
            """
            Test helper: overrides the grant_deadline and revocable_after timestamps on a test-mode Issuing dispute's provisional credit, allowing tests to simulate timer-driven status transitions without waiting for real regulatory deadlines to pass.
            """
            return cast(
                "Dispute",
                self.resource._request(
                    "post",
                    "/v1/test_helpers/issuing/disputes/{dispute}/provisional_credit".format(
                        dispute=sanitize_id(self.resource._data.get("id"))
                    ),
                    params=params,
                ),
            )

        @classmethod
        async def _cls_provisional_credit_async(
            cls,
            dispute: str,
            **params: Unpack["DisputeProvisionalCreditParams"],
        ) -> "Dispute":
            """
            Test helper: overrides the grant_deadline and revocable_after timestamps on a test-mode Issuing dispute's provisional credit, allowing tests to simulate timer-driven status transitions without waiting for real regulatory deadlines to pass.
            """
            return cast(
                "Dispute",
                await cls._static_request_async(
                    "post",
                    "/v1/test_helpers/issuing/disputes/{dispute}/provisional_credit".format(
                        dispute=sanitize_id(dispute)
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        async def provisional_credit_async(
            dispute: str, **params: Unpack["DisputeProvisionalCreditParams"]
        ) -> "Dispute":
            """
            Test helper: overrides the grant_deadline and revocable_after timestamps on a test-mode Issuing dispute's provisional credit, allowing tests to simulate timer-driven status transitions without waiting for real regulatory deadlines to pass.
            """
            ...

        @overload
        async def provisional_credit_async(
            self, **params: Unpack["DisputeProvisionalCreditParams"]
        ) -> "Dispute":
            """
            Test helper: overrides the grant_deadline and revocable_after timestamps on a test-mode Issuing dispute's provisional credit, allowing tests to simulate timer-driven status transitions without waiting for real regulatory deadlines to pass.
            """
            ...

        @class_method_variant("_cls_provisional_credit_async")
        async def provisional_credit_async(  # pyright: ignore[reportGeneralTypeIssues]
            self, **params: Unpack["DisputeProvisionalCreditParams"]
        ) -> "Dispute":
            """
            Test helper: overrides the grant_deadline and revocable_after timestamps on a test-mode Issuing dispute's provisional credit, allowing tests to simulate timer-driven status transitions without waiting for real regulatory deadlines to pass.
            """
            return cast(
                "Dispute",
                await self.resource._request_async(
                    "post",
                    "/v1/test_helpers/issuing/disputes/{dispute}/provisional_credit".format(
                        dispute=sanitize_id(self.resource._data.get("id"))
                    ),
                    params=params,
                ),
            )

        @classmethod
        def _cls_simulate_network_lifecycle_dispute_response(
            cls,
            dispute: str,
            **params: Unpack[
                "DisputeSimulateNetworkLifecycleDisputeResponseParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.dispute_response on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes.
            """
            return cast(
                "Dispute",
                cls._static_request(
                    "post",
                    "/v1/test_helpers/issuing/disputes/{dispute}/simulate_network_lifecycle_dispute_response".format(
                        dispute=sanitize_id(dispute)
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        def simulate_network_lifecycle_dispute_response(
            dispute: str,
            **params: Unpack[
                "DisputeSimulateNetworkLifecycleDisputeResponseParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.dispute_response on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes.
            """
            ...

        @overload
        def simulate_network_lifecycle_dispute_response(
            self,
            **params: Unpack[
                "DisputeSimulateNetworkLifecycleDisputeResponseParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.dispute_response on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes.
            """
            ...

        @class_method_variant(
            "_cls_simulate_network_lifecycle_dispute_response",
        )
        def simulate_network_lifecycle_dispute_response(  # pyright: ignore[reportGeneralTypeIssues]
            self,
            **params: Unpack[
                "DisputeSimulateNetworkLifecycleDisputeResponseParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.dispute_response on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes.
            """
            return cast(
                "Dispute",
                self.resource._request(
                    "post",
                    "/v1/test_helpers/issuing/disputes/{dispute}/simulate_network_lifecycle_dispute_response".format(
                        dispute=sanitize_id(self.resource._data.get("id"))
                    ),
                    params=params,
                ),
            )

        @classmethod
        async def _cls_simulate_network_lifecycle_dispute_response_async(
            cls,
            dispute: str,
            **params: Unpack[
                "DisputeSimulateNetworkLifecycleDisputeResponseParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.dispute_response on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes.
            """
            return cast(
                "Dispute",
                await cls._static_request_async(
                    "post",
                    "/v1/test_helpers/issuing/disputes/{dispute}/simulate_network_lifecycle_dispute_response".format(
                        dispute=sanitize_id(dispute)
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        async def simulate_network_lifecycle_dispute_response_async(
            dispute: str,
            **params: Unpack[
                "DisputeSimulateNetworkLifecycleDisputeResponseParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.dispute_response on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes.
            """
            ...

        @overload
        async def simulate_network_lifecycle_dispute_response_async(
            self,
            **params: Unpack[
                "DisputeSimulateNetworkLifecycleDisputeResponseParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.dispute_response on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes.
            """
            ...

        @class_method_variant(
            "_cls_simulate_network_lifecycle_dispute_response_async",
        )
        async def simulate_network_lifecycle_dispute_response_async(  # pyright: ignore[reportGeneralTypeIssues]
            self,
            **params: Unpack[
                "DisputeSimulateNetworkLifecycleDisputeResponseParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.dispute_response on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes.
            """
            return cast(
                "Dispute",
                await self.resource._request_async(
                    "post",
                    "/v1/test_helpers/issuing/disputes/{dispute}/simulate_network_lifecycle_dispute_response".format(
                        dispute=sanitize_id(self.resource._data.get("id"))
                    ),
                    params=params,
                ),
            )

        @classmethod
        def _cls_simulate_network_lifecycle_pre_arbitration_response(
            cls,
            dispute: str,
            **params: Unpack[
                "DisputeSimulateNetworkLifecyclePreArbitrationResponseParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.pre_arbitration_response on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes in the collaboration flow.
            """
            return cast(
                "Dispute",
                cls._static_request(
                    "post",
                    "/v1/test_helpers/issuing/disputes/{dispute}/simulate_network_lifecycle_pre_arbitration_response".format(
                        dispute=sanitize_id(dispute)
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        def simulate_network_lifecycle_pre_arbitration_response(
            dispute: str,
            **params: Unpack[
                "DisputeSimulateNetworkLifecyclePreArbitrationResponseParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.pre_arbitration_response on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes in the collaboration flow.
            """
            ...

        @overload
        def simulate_network_lifecycle_pre_arbitration_response(
            self,
            **params: Unpack[
                "DisputeSimulateNetworkLifecyclePreArbitrationResponseParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.pre_arbitration_response on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes in the collaboration flow.
            """
            ...

        @class_method_variant(
            "_cls_simulate_network_lifecycle_pre_arbitration_response",
        )
        def simulate_network_lifecycle_pre_arbitration_response(  # pyright: ignore[reportGeneralTypeIssues]
            self,
            **params: Unpack[
                "DisputeSimulateNetworkLifecyclePreArbitrationResponseParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.pre_arbitration_response on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes in the collaboration flow.
            """
            return cast(
                "Dispute",
                self.resource._request(
                    "post",
                    "/v1/test_helpers/issuing/disputes/{dispute}/simulate_network_lifecycle_pre_arbitration_response".format(
                        dispute=sanitize_id(self.resource._data.get("id"))
                    ),
                    params=params,
                ),
            )

        @classmethod
        async def _cls_simulate_network_lifecycle_pre_arbitration_response_async(
            cls,
            dispute: str,
            **params: Unpack[
                "DisputeSimulateNetworkLifecyclePreArbitrationResponseParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.pre_arbitration_response on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes in the collaboration flow.
            """
            return cast(
                "Dispute",
                await cls._static_request_async(
                    "post",
                    "/v1/test_helpers/issuing/disputes/{dispute}/simulate_network_lifecycle_pre_arbitration_response".format(
                        dispute=sanitize_id(dispute)
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        async def simulate_network_lifecycle_pre_arbitration_response_async(
            dispute: str,
            **params: Unpack[
                "DisputeSimulateNetworkLifecyclePreArbitrationResponseParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.pre_arbitration_response on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes in the collaboration flow.
            """
            ...

        @overload
        async def simulate_network_lifecycle_pre_arbitration_response_async(
            self,
            **params: Unpack[
                "DisputeSimulateNetworkLifecyclePreArbitrationResponseParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.pre_arbitration_response on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes in the collaboration flow.
            """
            ...

        @class_method_variant(
            "_cls_simulate_network_lifecycle_pre_arbitration_response_async",
        )
        async def simulate_network_lifecycle_pre_arbitration_response_async(  # pyright: ignore[reportGeneralTypeIssues]
            self,
            **params: Unpack[
                "DisputeSimulateNetworkLifecyclePreArbitrationResponseParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.pre_arbitration_response on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes in the collaboration flow.
            """
            return cast(
                "Dispute",
                await self.resource._request_async(
                    "post",
                    "/v1/test_helpers/issuing/disputes/{dispute}/simulate_network_lifecycle_pre_arbitration_response".format(
                        dispute=sanitize_id(self.resource._data.get("id"))
                    ),
                    params=params,
                ),
            )

        @classmethod
        def _cls_simulate_network_lifecycle_pre_arbitration_submission(
            cls,
            dispute: str,
            **params: Unpack[
                "DisputeSimulateNetworkLifecyclePreArbitrationSubmissionParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.pre_arbitration_submission on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes.
            """
            return cast(
                "Dispute",
                cls._static_request(
                    "post",
                    "/v1/test_helpers/issuing/disputes/{dispute}/simulate_network_lifecycle_pre_arbitration_submission".format(
                        dispute=sanitize_id(dispute)
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        def simulate_network_lifecycle_pre_arbitration_submission(
            dispute: str,
            **params: Unpack[
                "DisputeSimulateNetworkLifecyclePreArbitrationSubmissionParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.pre_arbitration_submission on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes.
            """
            ...

        @overload
        def simulate_network_lifecycle_pre_arbitration_submission(
            self,
            **params: Unpack[
                "DisputeSimulateNetworkLifecyclePreArbitrationSubmissionParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.pre_arbitration_submission on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes.
            """
            ...

        @class_method_variant(
            "_cls_simulate_network_lifecycle_pre_arbitration_submission",
        )
        def simulate_network_lifecycle_pre_arbitration_submission(  # pyright: ignore[reportGeneralTypeIssues]
            self,
            **params: Unpack[
                "DisputeSimulateNetworkLifecyclePreArbitrationSubmissionParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.pre_arbitration_submission on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes.
            """
            return cast(
                "Dispute",
                self.resource._request(
                    "post",
                    "/v1/test_helpers/issuing/disputes/{dispute}/simulate_network_lifecycle_pre_arbitration_submission".format(
                        dispute=sanitize_id(self.resource._data.get("id"))
                    ),
                    params=params,
                ),
            )

        @classmethod
        async def _cls_simulate_network_lifecycle_pre_arbitration_submission_async(
            cls,
            dispute: str,
            **params: Unpack[
                "DisputeSimulateNetworkLifecyclePreArbitrationSubmissionParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.pre_arbitration_submission on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes.
            """
            return cast(
                "Dispute",
                await cls._static_request_async(
                    "post",
                    "/v1/test_helpers/issuing/disputes/{dispute}/simulate_network_lifecycle_pre_arbitration_submission".format(
                        dispute=sanitize_id(dispute)
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        async def simulate_network_lifecycle_pre_arbitration_submission_async(
            dispute: str,
            **params: Unpack[
                "DisputeSimulateNetworkLifecyclePreArbitrationSubmissionParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.pre_arbitration_submission on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes.
            """
            ...

        @overload
        async def simulate_network_lifecycle_pre_arbitration_submission_async(
            self,
            **params: Unpack[
                "DisputeSimulateNetworkLifecyclePreArbitrationSubmissionParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.pre_arbitration_submission on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes.
            """
            ...

        @class_method_variant(
            "_cls_simulate_network_lifecycle_pre_arbitration_submission_async",
        )
        async def simulate_network_lifecycle_pre_arbitration_submission_async(  # pyright: ignore[reportGeneralTypeIssues]
            self,
            **params: Unpack[
                "DisputeSimulateNetworkLifecyclePreArbitrationSubmissionParams"
            ],
        ) -> "Dispute":
            """
            Test helper: populates network_lifecycle.pre_arbitration_submission on a test-mode Visa Issuing Dispute using placeholder file tokens. Only supported for Visa disputes.
            """
            return cast(
                "Dispute",
                await self.resource._request_async(
                    "post",
                    "/v1/test_helpers/issuing/disputes/{dispute}/simulate_network_lifecycle_pre_arbitration_submission".format(
                        dispute=sanitize_id(self.resource._data.get("id"))
                    ),
                    params=params,
                ),
            )

    @property
    def test_helpers(self):
        return self.TestHelpers(self)

    _inner_class_types = {
        "crypto_transactions": CryptoTransaction,
        "evidence": Evidence,
        "network_lifecycle": NetworkLifecycle,
        "provisional_credit": ProvisionalCredit,
        "redaction": Redaction,
        "treasury": Treasury,
    }


Dispute.TestHelpers._resource_cls = Dispute



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_object import StripeObject
from typing import ClassVar, Optional
from typing_extensions import Literal


class RecipientVerification(StripeObject):
    """
    RecipientVerification represents a verification of recipient you intend to send funds to.
    """

    OBJECT_NAME: ClassVar[
        Literal["v2.money_management.recipient_verification"]
    ] = "v2.money_management.recipient_verification"

    class MatchResultDetails(StripeObject):
        matched_name: Optional[str]
        """
        The account name associated with the bank account as provided by the VoP provider, only present if there is a match or close match.
        """
        message: str
        """
        A message describing the match result.
        """
        provided_name: str
        """
        The name associated with the provided recipient.
        """

    class StatusTransitions(StripeObject):
        acknowledged_at: Optional[str]
        """
        Timestamp describing when a RecipientVerification changed status to `acknowledged`.
        Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
        """
        consumed_at: Optional[str]
        """
        Timestamp describing when a RecipientVerification changed status to `consumed`.
        Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
        """

    consumed_by: Optional[str]
    """
    The OBP/OBT ID that consumed this verification, present if one is successfully created.
    """
    created: str
    """
    Time at which the RecipientVerification was created.
    Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
    """
    expires_at: str
    """
    Time at which the RecipientVerification expires, 5 minutes after the creation.
    Represented as a RFC 3339 date & time UTC value in millisecond precision, for example: 2022-09-18T13:22:18.123Z.
    """
    id: str
    """
    The ID of the RecipientVerification.
    """
    livemode: bool
    """
    Has the value `true` if the object exists in live mode or the value `false` if the object exists in test mode.
    """
    match_result: Literal["close_match", "match", "no_match", "unavailable"]
    """
    Closed Enum. Match level of the RecipientVerification: `match`, `close_match`, `no_match`, `unavailable`.
    """
    match_result_details: MatchResultDetails
    """
    Details for the match result.
    """
    object: Literal["v2.money_management.recipient_verification"]
    """
    String representing the object's type. Objects of the same type share the same value of the object field.
    """
    status: Literal[
        "acknowledged",
        "awaiting_acknowledgement",
        "consumed",
        "expired",
        "verified",
    ]
    """
    Closed Enum. Current status of the RecipientVerification: `verified`, `consumed`, `expired`, `awaiting_acknowledgement`, `acknowledged`.
    """
    status_transitions: Optional[StatusTransitions]
    """
    Hash containing timestamps of when the object transitioned to a particular status.
    """
    _inner_class_types = {
        "match_result_details": MatchResultDetails,
        "status_transitions": StatusTransitions,
    }



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._api_resource import APIResource
from stripe._expandable_field import ExpandableField
from stripe._stripe_object import StripeObject, UntypedStripeObject
from stripe._test_helpers import APIResourceTestHelpers
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, List, Optional, cast, overload
from typing_extensions import Literal, Type, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._charge import Charge
    from stripe._payment_method import PaymentMethod
    from stripe._setup_attempt import SetupAttempt
    from stripe.params.shared_payment._granted_token_create_params import (
        GrantedTokenCreateParams,
    )
    from stripe.params.shared_payment._granted_token_retrieve_params import (
        GrantedTokenRetrieveParams,
    )
    from stripe.params.shared_payment._granted_token_revoke_params import (
        GrantedTokenRevokeParams,
    )


class GrantedToken(APIResource["GrantedToken"]):
    """
    SharedPaymentGrantedToken is the view-only resource of a SharedPaymentIssuedToken, which is a limited-use reference to a PaymentMethod.
    When another Stripe merchant shares a SharedPaymentIssuedToken with you, you can view attributes of the shared token using the SharedPaymentGrantedToken API, and use it with a PaymentIntent.
    """

    OBJECT_NAME: ClassVar[Literal["shared_payment.granted_token"]] = (
        "shared_payment.granted_token"
    )

    class AgentDetails(StripeObject):
        network_business_profile: Optional[str]
        """
        The Stripe Profile ID of the agent that issued this SharedPaymentGrantedToken.
        """

    class PaymentMethodDetails(StripeObject):
        class AcssDebit(StripeObject):
            account_number: Optional[str]
            """
            Account number of the bank account.
            """
            bank_name: Optional[str]
            """
            Name of the bank associated with the bank account.
            """
            fingerprint: Optional[str]
            """
            Uniquely identifies this particular bank account. You can use this attribute to check whether two bank accounts are the same.
            """
            institution_number: Optional[str]
            """
            Institution number of the bank account.
            """
            last4: Optional[str]
            """
            Last four digits of the bank account number.
            """
            transit_number: Optional[str]
            """
            Transit number of the bank account.
            """

        class Affirm(StripeObject):
            pass

        class AfterpayClearpay(StripeObject):
            pass

        class Alipay(StripeObject):
            pass

        class Alma(StripeObject):
            pass

        class AmazonPay(StripeObject):
            pass

        class AuBecsDebit(StripeObject):
            bsb_number: Optional[str]
            """
            Six-digit number identifying bank and branch associated with this bank account.
            """
            fingerprint: Optional[str]
            """
            Uniquely identifies this particular bank account. You can use this attribute to check whether two bank accounts are the same.
            """
            last4: Optional[str]
            """
            Last four digits of the bank account number.
            """

        class BacsDebit(StripeObject):
            fingerprint: Optional[str]
            """
            Uniquely identifies this particular bank account. You can use this attribute to check whether two bank accounts are the same.
            """
            last4: Optional[str]
            """
            Last four digits of the bank account number.
            """
            sort_code: Optional[str]
            """
            Sort code of the bank account. (e.g., `10-20-30`)
            """

        class Bancontact(StripeObject):
            pass

        class Billie(StripeObject):
            pass

        class BillingDetails(StripeObject):
            class Address(StripeObject):
                city: Optional[str]
                """
                City, district, suburb, town, or village.
                """
                country: Optional[str]
                """
                Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
                """
                line1: Optional[str]
                """
                Address line 1, such as the street, PO Box, or company name.
                """
                line2: Optional[str]
                """
                Address line 2, such as the apartment, suite, unit, or building.
                """
                postal_code: Optional[str]
                """
                ZIP or postal code.
                """
                state: Optional[str]
                """
                State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
                """

            address: Optional[Address]
            """
            Billing address.
            """
            email: Optional[str]
            """
            Email address.
            """
            name: Optional[str]
            """
            Full name.
            """
            phone: Optional[str]
            """
            Billing phone number (including extension).
            """
            tax_id: Optional[str]
            """
            Taxpayer identification number. Used only for transactions between LATAM buyers and non-LATAM sellers.
            """
            _inner_class_types = {"address": Address}

        class Bizum(StripeObject):
            buyer_id: Optional[str]
            """
            A unique identifier for the buyer as determined by the local payment processor.
            """

        class Blik(StripeObject):
            buyer_id: Optional[str]
            """
            A unique and immutable identifier assigned by BLIK to every buyer.
            """

        class Boleto(StripeObject):
            tax_id: str
            """
            Uniquely identifies the customer tax id (CNPJ or CPF)
            """

        class Card(StripeObject):
            class Checks(StripeObject):
                address_line1_check: Optional[str]
                """
                If a address line1 was provided, results of the check, one of `pass`, `fail`, `unavailable`, or `unchecked`.
                """
                address_postal_code_check: Optional[str]
                """
                If a address postal code was provided, results of the check, one of `pass`, `fail`, `unavailable`, or `unchecked`.
                """
                cvc_check: Optional[str]
                """
                If a CVC was provided, results of the check, one of `pass`, `fail`, `unavailable`, or `unchecked`.
                """

            class Networks(StripeObject):
                available: List[str]
                """
                All networks available for selection via [payment_method_options.card.network](https://docs.stripe.com/api/payment_intents/confirm#confirm_payment_intent-payment_method_options-card-network).
                """
                preferred: Optional[str]
                """
                The preferred network for co-branded cards. Can be `cartes_bancaires`, `mastercard`, `visa` or `invalid_preference` if requested network is not valid for the card.
                """

            class Wallet(StripeObject):
                class AmexExpressCheckout(StripeObject):
                    pass

                class ApplePay(StripeObject):
                    pass

                class GooglePay(StripeObject):
                    pass

                class Link(StripeObject):
                    pass

                class Masterpass(StripeObject):
                    class BillingAddress(StripeObject):
                        city: Optional[str]
                        """
                        City, district, suburb, town, or village.
                        """
                        country: Optional[str]
                        """
                        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
                        """
                        line1: Optional[str]
                        """
                        Address line 1, such as the street, PO Box, or company name.
                        """
                        line2: Optional[str]
                        """
                        Address line 2, such as the apartment, suite, unit, or building.
                        """
                        postal_code: Optional[str]
                        """
                        ZIP or postal code.
                        """
                        state: Optional[str]
                        """
                        State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
                        """

                    class ShippingAddress(StripeObject):
                        city: Optional[str]
                        """
                        City, district, suburb, town, or village.
                        """
                        country: Optional[str]
                        """
                        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
                        """
                        line1: Optional[str]
                        """
                        Address line 1, such as the street, PO Box, or company name.
                        """
                        line2: Optional[str]
                        """
                        Address line 2, such as the apartment, suite, unit, or building.
                        """
                        postal_code: Optional[str]
                        """
                        ZIP or postal code.
                        """
                        state: Optional[str]
                        """
                        State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
                        """

                    billing_address: Optional[BillingAddress]
                    """
                    Owner's verified billing address. Values are verified or provided by the wallet directly (if supported) at the time of authorization or settlement. They cannot be set or mutated.
                    """
                    email: Optional[str]
                    """
                    Owner's verified email. Values are verified or provided by the wallet directly (if supported) at the time of authorization or settlement. They cannot be set or mutated.
                    """
                    name: Optional[str]
                    """
                    Owner's verified full name. Values are verified or provided by the wallet directly (if supported) at the time of authorization or settlement. They cannot be set or mutated.
                    """
                    shipping_address: Optional[ShippingAddress]
                    """
                    Owner's verified shipping address. Values are verified or provided by the wallet directly (if supported) at the time of authorization or settlement. They cannot be set or mutated.
                    """
                    _inner_class_types = {
                        "billing_address": BillingAddress,
                        "shipping_address": ShippingAddress,
                    }

                class SamsungPay(StripeObject):
                    pass

                class VisaCheckout(StripeObject):
                    class BillingAddress(StripeObject):
                        city: Optional[str]
                        """
                        City, district, suburb, town, or village.
                        """
                        country: Optional[str]
                        """
                        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
                        """
                        line1: Optional[str]
                        """
                        Address line 1, such as the street, PO Box, or company name.
                        """
                        line2: Optional[str]
                        """
                        Address line 2, such as the apartment, suite, unit, or building.
                        """
                        postal_code: Optional[str]
                        """
                        ZIP or postal code.
                        """
                        state: Optional[str]
                        """
                        State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
                        """

                    class ShippingAddress(StripeObject):
                        city: Optional[str]
                        """
                        City, district, suburb, town, or village.
                        """
                        country: Optional[str]
                        """
                        Two-letter country code ([ISO 3166-1 alpha-2](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2)).
                        """
                        line1: Optional[str]
                        """
                        Address line 1, such as the street, PO Box, or company name.
                        """
                        line2: Optional[str]
                        """
                        Address line 2, such as the apartment, suite, unit, or building.
                        """
                        postal_code: Optional[str]
                        """
                        ZIP or postal code.
                        """
                        state: Optional[str]
                        """
                        State, county, province, or region ([ISO 3166-2](https://en.wikipedia.org/wiki/ISO_3166-2)).
                        """

                    billing_address: Optional[BillingAddress]
                    """
                    Owner's verified billing address. Values are verified or provided by the wallet directly (if supported) at the time of authorization or settlement. They cannot be set or mutated.
                    """
                    email: Optional[str]
                    """
                    Owner's verified email. Values are verified or provided by the wallet directly (if supported) at the time of authorization or settlement. They cannot be set or mutated.
                    """
                    name: Optional[str]
                    """
                    Owner's verified full name. Values are verified or provided by the wallet directly (if supported) at the time of authorization or settlement. They cannot be set or mutated.
                    """
                    shipping_address: Optional[ShippingAddress]
                    """
                    Owner's verified shipping address. Values are verified or provided by the wallet directly (if supported) at the time of authorization or settlement. They cannot be set or mutated.
                    """
                    _inner_class_types = {
                        "billing_address": BillingAddress,
                        "shipping_address": ShippingAddress,
                    }

                amex_express_checkout: Optional[AmexExpressCheckout]
                apple_pay: Optional[ApplePay]
                dynamic_last4: Optional[str]
                """
                (For tokenized numbers only.) The last four digits of the device account number.
                """
                google_pay: Optional[GooglePay]
                link: Optional[Link]
                masterpass: Optional[Masterpass]
                samsung_pay: Optional[SamsungPay]
                type: Literal[
                    "amex_express_checkout",
                    "apple_pay",
                    "google_pay",
                    "link",
                    "masterpass",
                    "samsung_pay",
                    "visa_checkout",
                ]
                """
                The type of the card wallet, one of `amex_express_checkout`, `apple_pay`, `google_pay`, `masterpass`, `samsung_pay`, `visa_checkout`, or `link`. An additional hash is included on the Wallet subhash with a name matching this value. It contains additional information specific to the card wallet type.
                """
                visa_checkout: Optional[VisaCheckout]
                _inner_class_types = {
                    "amex_express_checkout": AmexExpressCheckout,
                    "apple_pay": ApplePay,
                    "google_pay": GooglePay,
                    "link": Link,
                    "masterpass": Masterpass,
                    "samsung_pay": SamsungPay,
                    "visa_checkout": VisaCheckout,
                }

            brand: str
            """
            Card brand. Can be `amex`, `cartes_bancaires`, `diners`, `discover`, `eftpos_au`, `jcb`, `link`, `mastercard`, `unionpay`, `visa` or `unknown`.
            """
            checks: Optional[Checks]
            """
            Checks on Card address and CVC if provided.
            """
            country: Optional[str]
            """
            Two-letter ISO code representing the country of the card. You could use this attribute to get a sense of the international breakdown of cards you've collected.
            """
            description: Optional[str]
            """
            A high-level description of the type of cards issued in this range. (For internal use only and not typically available in standard API requests.)
            """
            display_brand: Optional[str]
            """
            The brand to use when displaying the card, this accounts for customer's brand choice on dual-branded cards. Can be `american_express`, `cartes_bancaires`, `diners_club`, `discover`, `eftpos_australia`, `interac`, `jcb`, `mastercard`, `union_pay`, `visa`, or `other` and may contain more values in the future.
            """
            exp_month: int
            """
            Two-digit number representing the card's expiration month.
            """
            exp_year: int
            """
            Four-digit number representing the card's expiration year.
            """
            fingerprint: Optional[str]
            """
            Uniquely identifies this particular card number. You can use this attribute to check whether two customers who've signed up with you are using the same card number, for example. For payment methods that tokenize card information (Apple Pay, Google Pay), the tokenized number might be provided instead of the underlying card number.

            *As of May 1, 2021, card fingerprint in India for Connect changed to allow two fingerprints for the same card---one for India and one for the rest of the world.*
            """
            funding: str
            """
            Card funding type. Can be `credit`, `debit`, `prepaid`, or `unknown`.
            """
            iin: Optional[str]
            """
            Issuer identification number of the card. (For internal use only and not typically available in standard API requests.)
            """
            issuer: Optional[str]
            """
            The name of the card's issuing bank. (For internal use only and not typically available in standard API requests.)
            """
            last4: str
            """
            The last four digits of the card.
            """
            networks: Optional[Networks]
            """
            Contains information about card networks that can be used to process the payment.
            """
            wallet: Optional[Wallet]
            """
            If this Card is part of a card wallet, this contains the details of the card wallet.
            """
            _inner_class_types = {
                "checks": Checks,
                "networks": Networks,
                "wallet": Wallet,
            }

        class CardPresent(StripeObject):
            class Networks(StripeObject):
                available: List[str]
                """
                All networks available for selection via [payment_method_options.card.network](https://docs.stripe.com/api/payment_intents/confirm#confirm_payment_intent-payment_method_options-card-network).
                """
                preferred: Optional[str]
                """
                The preferred network for the card.
                """

            class Offline(StripeObject):
                stored_at: Optional[int]
                """
                Time at which the payment was collected while offline
                """
                type: Optional[Literal["deferred"]]
                """
                The method used to process this payment method offline. Only deferred is allowed.
                """

            class Wallet(StripeObject):
                type: Literal[
                    "apple_pay", "google_pay", "samsung_pay", "unknown"
                ]
                """
                The type of mobile wallet, one of `apple_pay`, `google_pay`, `samsung_pay`, or `unknown`.
                """

            brand: Optional[str]
            """
            Card brand. Can be `amex`, `cartes_bancaires`, `diners`, `discover`, `eftpos_au`, `jcb`, `link`, `mastercard`, `unionpay`, `visa` or `unknown`.
            """
            brand_product: Optional[str]
            """
            The [product code](https://stripe.com/docs/card-product-codes) that identifies the specific program or product associated with a card.
            """
            cardholder_name: Optional[str]
            """
            The cardholder name as read from the card, in [ISO 7813](https://en.wikipedia.org/wiki/ISO/IEC_7813) format. May include alphanumeric characters, special characters and first/last name separator (`/`). In some cases, the cardholder name may not be available depending on how the issuer has configured the card. Cardholder name is typically not available on swipe or contactless payments, such as those made with Apple Pay and Google Pay.
            """
            country: Optional[str]
            """
            Two-letter ISO code representing the country of the card. You could use this attribute to get a sense of the international breakdown of cards you've collected.
            """
            description: Optional[str]
            """
            A high-level description of the type of cards issued in this range. (For internal use only and not typically available in standard API requests.)
            """
            exp_month: int
            """
            Two-digit number representing the card's expiration month.
            """
            exp_year: int
            """
            Four-digit number representing the card's expiration year.
            """
            fingerprint: Optional[str]
            """
            Uniquely identifies this particular card number. You can use this attribute to check whether two customers who've signed up with you are using the same card number, for example. For payment methods that tokenize card information (Apple Pay, Google Pay), the tokenized number might be provided instead of the underlying card number.

            *As of May 1, 2021, card fingerprint in India for Connect changed to allow two fingerprints for the same card---one for India and one for the rest of the world.*
            """
            funding: Optional[str]
            """
            Card funding type. Can be `credit`, `debit`, `prepaid`, or `unknown`.
            """
            iin: Optional[str]
            """
            Issuer identification number of the card. (For internal use only and not typically available in standard API requests.)
            """
            issuer: Optional[str]
            """
            The name of the card's issuing bank. (For internal use only and not typically available in standard API requests.)
            """
            last4: Optional[str]
            """
            The last four digits of the card.
            """
            networks: Optional[Networks]
            """
            Contains information about card networks that can be used to process the payment.
            """
            offline: Optional[Offline]
            """
            Details about payment methods collected offline.
            """
            preferred_locales: Optional[List[str]]
            """
            The languages that the issuing bank recommends using for localizing any customer-facing text, as read from the card. Referenced from EMV tag 5F2D, data encoded on the card's chip.
            """
            read_method: Optional[
                Literal[
                    "contact_emv",
                    "contactless_emv",
                    "contactless_magstripe_mode",
                    "magnetic_stripe_fallback",
                    "magnetic_stripe_track2",
                ]
            ]
            """
            How card details were read in this transaction.
            """
            wallet: Optional[Wallet]
            _inner_class_types = {
                "networks": Networks,
                "offline": Offline,
                "wallet": Wallet,
            }

        class Cashapp(StripeObject):
            buyer_id: Optional[str]
            """
            A unique and immutable identifier assigned by Cash App to every buyer.
            """
            cashtag: Optional[str]
            """
            A public identifier for buyers using Cash App.
            """

        class Crypto(StripeObject):
            pass

        class CustomerBalance(StripeObject):
            pass

        class Eps(StripeObject):
            bank: Optional[
                Literal[
                    "arzte_und_apotheker_bank",
                    "austrian_anadi_bank_ag",
                    "bank_austria",
                    "bankhaus_carl_spangler",
                    "bankhaus_schelhammer_und_schattera_ag",
                    "bawag_psk_ag",
                    "bks_bank_ag",
                    "brull_kallmus_bank_ag",
                    "btv_vier_lander_bank",
                    "capital_bank_grawe_gruppe_ag",
                    "deutsche_bank_ag",
                    "dolomitenbank",
                    "easybank_ag",
                    "erste_bank_und_sparkassen",
                    "hypo_alpeadriabank_international_ag",
                    "hypo_bank_burgenland_aktiengesellschaft",
                    "hypo_noe_lb_fur_niederosterreich_u_wien",
                    "hypo_oberosterreich_salzburg_steiermark",
                    "hypo_tirol_bank_ag",
                    "hypo_vorarlberg_bank_ag",
                    "marchfelder_bank",
                    "oberbank_ag",
                    "raiffeisen_bankengruppe_osterreich",
                    "schoellerbank_ag",
                    "sparda_bank_wien",
                    "volksbank_gruppe",
                    "volkskreditbank_ag",
                    "vr_bank_braunau",
                ]
            ]
            """
            The customer's bank. Should be one of `arzte_und_apotheker_bank`, `austrian_anadi_bank_ag`, `bank_austria`, `bankhaus_carl_spangler`, `bankhaus_schelhammer_und_schattera_ag`, `bawag_psk_ag`, `bks_bank_ag`, `brull_kallmus_bank_ag`, `btv_vier_lander_bank`, `capital_bank_grawe_gruppe_ag`, `deutsche_bank_ag`, `dolomitenbank`, `easybank_ag`, `erste_bank_und_sparkassen`, `hypo_alpeadriabank_international_ag`, `hypo_noe_lb_fur_niederosterreich_u_wien`, `hypo_oberosterreich_salzburg_steiermark`, `hypo_tirol_bank_ag`, `hypo_vorarlberg_bank_ag`, `hypo_bank_burgenland_aktiengesellschaft`, `marchfelder_bank`, `oberbank_ag`, `raiffeisen_bankengruppe_osterreich`, `schoellerbank_ag`, `sparda_bank_wien`, `volksbank_gruppe`, `volkskreditbank_ag`, or `vr_bank_braunau`.
            """

        class Fpx(StripeObject):
            account_holder_type: Optional[Literal["company", "individual"]]
            """
            Account holder type, if provided. Can be one of `individual` or `company`.
            """
            bank: Literal[
                "affin_bank",
                "agrobank",
                "alliance_bank",
                "ambank",
                "bank_islam",
                "bank_muamalat",
                "bank_of_china",
                "bank_rakyat",
                "bsn",
                "cimb",
                "deutsche_bank",
                "hong_leong_bank",
                "hsbc",
                "kfh",
                "maybank2e",
                "maybank2u",
                "ocbc",
                "pb_enterprise",
                "public_bank",
                "rhb",
                "standard_chartered",
                "uob",
            ]
            """
            The customer's bank, if provided. Can be one of `affin_bank`, `agrobank`, `alliance_bank`, `ambank`, `bank_islam`, `bank_muamalat`, `bank_rakyat`, `bsn`, `cimb`, `hong_leong_bank`, `hsbc`, `kfh`, `maybank2u`, `ocbc`, `public_bank`, `rhb`, `standard_chartered`, `uob`, `deutsche_bank`, `maybank2e`, `pb_enterprise`, or `bank_of_china`.
            """

        class GiftCard(StripeObject):
            brand: Literal["fiserv_valuelink", "givex", "svs"]
            """
            The brand of the gift card.
            """
            exp_month: Optional[int]
            """
            The expiration month of the gift card.
            """
            exp_year: Optional[int]
            """
            The expiration year of the gift card.
            """
            fingerprint: Optional[str]
            """
            Uniquely identifies the gift card.
            """
            last4: Optional[str]
            """
            The last four digits of the gift card number.
            """

        class Giropay(StripeObject):
            pass

        class Gopay(StripeObject):
            pass

        class Grabpay(StripeObject):
            pass

        class IdBankTransfer(StripeObject):
            bank: Optional[Literal["bca", "bni", "bri", "cimb", "permata"]]
            bank_code: Optional[str]
            bank_name: Optional[str]
            display_name: Optional[str]

        class Ideal(StripeObject):
            bank: Optional[
                Literal[
                    "abn_amro",
                    "adyen",
                    "asn_bank",
                    "bunq",
                    "buut",
                    "finom",
                    "handelsbanken",
                    "ing",
                    "knab",
                    "mollie",
                    "moneyou",
                    "n26",
                    "nn",
                    "rabobank",
                    "regiobank",
                    "revolut",
                    "sns_bank",
                    "triodos_bank",
                    "van_lanschot",
                    "yoursafe",
                ]
            ]
            """
            The customer's bank, if provided. Can be one of `abn_amro`, `adyen`, `asn_bank`, `bunq`, `buut`, `finom`, `handelsbanken`, `ing`, `knab`, `mollie`, `moneyou`, `n26`, `nn`, `rabobank`, `regiobank`, `revolut`, `sns_bank`, `triodos_bank`, `van_lanschot`, or `yoursafe`.
            """
            bic: Optional[
                Literal[
                    "ABNANL2A",
                    "ADYBNL2A",
                    "ASNBNL21",
                    "BITSNL2A",
                    "BUNQNL2A",
                    "BUUTNL2A",
                    "FNOMNL22",
                    "FVLBNL22",
                    "HANDNL2A",
                    "INGBNL2A",
                    "KNABNL2H",
                    "MLLENL2A",
                    "MOYONL21",
                    "NNBANL2G",
                    "NTSBDEB1",
                    "RABONL2U",
                    "RBRBNL21",
                    "REVOIE23",
                    "REVOLT21",
                    "SNSBNL2A",
                    "TRIONL2U",
                ]
            ]
            """
            The Bank Identifier Code of the customer's bank, if the bank was provided.
            """

        class InteracPresent(StripeObject):
            class Networks(StripeObject):
                available: List[str]
                """
                All networks available for selection via [payment_method_options.card.network](https://docs.stripe.com/api/payment_intents/confirm#confirm_payment_intent-payment_method_options-card-network).
                """
                preferred: Optional[str]
                """
                The preferred network for the card.
                """

            brand: Optional[str]
            """
            Card brand. Can be `interac`, `mastercard` or `visa`.
            """
            cardholder_name: Optional[str]
            """
            The cardholder name as read from the card, in [ISO 7813](https://en.wikipedia.org/wiki/ISO/IEC_7813) format. May include alphanumeric characters, special characters and first/last name separator (`/`). In some cases, the cardholder name may not be available depending on how the issuer has configured the card. Cardholder name is typically not available on swipe or contactless payments, such as those made with Apple Pay and Google Pay.
            """
            country: Optional[str]
            """
            Two-letter ISO code representing the country of the card. You could use this attribute to get a sense of the international breakdown of cards you've collected.
            """
            description: Optional[str]
            """
            A high-level description of the type of cards issued in this range. (For internal use only and not typically available in standard API requests.)
            """
            exp_month: int
            """
            Two-digit number representing the card's expiration month.
            """
            exp_year: int
            """
            Four-digit number representing the card's expiration year.
            """
            fingerprint: Optional[str]
            """
            Uniquely identifies this particular card number. You can use this attribute to check whether two customers who've signed up with you are using the same card number, for example. For payment methods that tokenize card information (Apple Pay, Google Pay), the tokenized number might be provided instead of the underlying card number.

            *As of May 1, 2021, card fingerprint in India for Connect changed to allow two fingerprints for the same card---one for India and one for the rest of the world.*
            """
            funding: Optional[str]
            """
            Card funding type. Can be `credit`, `debit`, `prepaid`, or `unknown`.
            """
            iin: Optional[str]
            """
            Issuer identification number of the card. (For internal use only and not typically available in standard API requests.)
            """
            issuer: Optional[str]
            """
            The name of the card's issuing bank. (For internal use only and not typically available in standard API requests.)
            """
            last4: Optional[str]
            """
            The last four digits of the card.
            """
            networks: Optional[Networks]
            """
            Contains information about card networks that can be used to process the payment.
            """
            preferred_locales: Optional[List[str]]
            """
            The languages that the issuing bank recommends using for localizing any customer-facing text, as read from the card. Referenced from EMV tag 5F2D, data encoded on the card's chip.
            """
            read_method: Optional[
                Literal[
                    "contact_emv",
                    "contactless_emv",
                    "contactless_magstripe_mode",
                    "magnetic_stripe_fallback",
                    "magnetic_stripe_track2",
                ]
            ]
            """
            How card details were read in this transaction.
            """
            _inner_class_types = {"networks": Networks}

        class KakaoPay(StripeObject):
            pass

        class Klarna(StripeObject):
            class Dob(StripeObject):
                day: Optional[int]
                """
                The day of birth, between 1 and 31.
                """
                month: Optional[int]
                """
                The month of birth, between 1 and 12.
                """
                year: Optional[int]
                """
                The four-digit year of birth.
                """

            dob: Optional[Dob]
            """
            The customer's date of birth, if provided.
            """
            _inner_class_types = {"dob": Dob}

        class Konbini(StripeObject):
            pass

        class KrCard(StripeObject):
            brand: Optional[
                Literal[
                    "bc",
                    "citi",
                    "hana",
                    "hyundai",
                    "jeju",
                    "jeonbuk",
                    "kakaobank",
                    "kbank",
                    "kdbbank",
                    "kookmin",
                    "kwangju",
                    "lotte",
                    "mg",
                    "nh",
                    "post",
                    "samsung",
                    "savingsbank",
                    "shinhan",
                    "shinhyup",
                    "suhyup",
                    "tossbank",
                    "woori",
                ]
            ]
            """
            The local credit or debit card brand.
            """
            last4: Optional[str]
            """
            The last four digits of the card. This may not be present for American Express cards.
            """

        class Link(StripeObject):
            email: Optional[str]
            """
            Account owner's email address.
            """
            persistent_token: Optional[str]
            """
            [Deprecated] This is a legacy parameter that no longer has any function.
            """

        class MbWay(StripeObject):
            pass

        class Mobilepay(StripeObject):
            pass

        class Multibanco(StripeObject):
            pass

        class NaverPay(StripeObject):
            buyer_id: Optional[str]
            """
            Uniquely identifies this particular Naver Pay account. You can use this attribute to check whether two Naver Pay accounts are the same.
            """
            funding: Literal["card", "points"]
            """
            Whether to fund this transaction with Naver Pay points or a card.
            """

        class NzBankAccount(StripeObject):
            account_holder_name: Optional[str]
            """
            The name on the bank account. Only present if the account holder name is different from the name of the authorized signatory collected in the PaymentMethod's billing details.
            """
            bank_code: str
            """
            The numeric code for the bank account's bank.
            """
            bank_name: str
            """
            The name of the bank.
            """
            branch_code: str
            """
            The numeric code for the bank account's bank branch.
            """
            last4: str
            """
            Last four digits of the bank account number.
            """
            suffix: Optional[str]
            """
            The suffix of the bank account number.
            """

        class Oxxo(StripeObject):
            pass

        class P24(StripeObject):
            bank: Optional[
                Literal[
                    "alior_bank",
                    "bank_millennium",
                    "bank_nowy_bfg_sa",
                    "bank_pekao_sa",
                    "banki_spbdzielcze",
                    "blik",
                    "bnp_paribas",
                    "boz",
                    "citi_handlowy",
                    "credit_agricole",
                    "envelobank",
                    "etransfer_pocztowy24",
                    "getin_bank",
                    "ideabank",
                    "ing",
                    "inteligo",
                    "mbank_mtransfer",
                    "nest_przelew",
                    "noble_pay",
                    "pbac_z_ipko",
                    "plus_bank",
                    "santander_przelew24",
                    "tmobile_usbugi_bankowe",
                    "toyota_bank",
                    "velobank",
                    "volkswagen_bank",
                ]
            ]
            """
            The customer's bank, if provided.
            """

        class PayByBank(StripeObject):
            pass

        class Payco(StripeObject):
            pass

        class Paynow(StripeObject):
            pass

        class Paypal(StripeObject):
            country: Optional[str]
            """
            Two-letter ISO code representing the buyer's country. Values are provided by PayPal directly (if supported) at the time of authorization or settlement. They cannot be set or mutated.
            """
            fingerprint: Optional[str]
            """
            Uniquely identifies this particular PayPal account. You can use this attribute to check whether two PayPal accounts are the same.
            """
            payer_email: Optional[str]
            """
            Owner's email. Values are provided by PayPal directly
            (if supported) at the time of authorization or settlement. They cannot be set or mutated.
            """
            payer_id: Optional[str]
            """
            PayPal account PayerID. This identifier uniquely identifies the PayPal customer.
            """
            verified_email: Optional[str]
            """
            Owner's verified email. Values are verified or provided by PayPal directly
            (if supported) at the time of authorization or settlement. They cannot be set or mutated.
            """

        class Paypay(StripeObject):
            pass

        class Payto(StripeObject):
            bsb_number: Optional[str]
            """
            Bank-State-Branch number of the bank account.
            """
            last4: Optional[str]
            """
            Last four digits of the bank account number.
            """
            pay_id: Optional[str]
            """
            The PayID alias for the bank account.
            """

        class Pix(StripeObject):
            fingerprint: Optional[str]
            """
            Uniquely identifies this particular Pix account. You can use this attribute to check whether two Pix accounts are the same.
            """

        class Promptpay(StripeObject):
            pass

        class Qris(StripeObject):
            pass

        class Rechnung(StripeObject):
            class Dob(StripeObject):
                day: int
                """
                The day of birth, between 1 and 31.
                """
                month: int
                """
                The month of birth, between 1 and 12.
                """
                year: int
                """
                The four-digit year of birth.
                """

            dob: Optional[Dob]
            _inner_class_types = {"dob": Dob}

        class RevolutPay(StripeObject):
            pass

        class SamsungPay(StripeObject):
            pass

        class Satispay(StripeObject):
            pass

        class Scalapay(StripeObject):
            pass

        class SepaDebit(StripeObject):
            class GeneratedFrom(StripeObject):
                charge: Optional[ExpandableField["Charge"]]
                """
                The ID of the Charge that generated this PaymentMethod, if any.
                """
                payment_method: Optional[ExpandableField["PaymentMethod"]]
                """
                The ID of the PaymentMethod that generated this PaymentMethod, if any.
                """
                setup_attempt: Optional[ExpandableField["SetupAttempt"]]
                """
                The ID of the SetupAttempt that generated this PaymentMethod, if any.
                """

            bank_code: Optional[str]
            """
            Bank code of bank associated with the bank account.
            """
            branch_code: Optional[str]
            """
            Branch code of bank associated with the bank account.
            """
            country: Optional[str]
            """
            Two-letter ISO code representing the country the bank account is located in.
            """
            fingerprint: Optional[str]
            """
            Uniquely identifies this particular bank account. You can use this attribute to check whether two bank accounts are the same.
            """
            generated_from: Optional[GeneratedFrom]
            """
            Information about the object that generated this PaymentMethod.
            """
            last4: Optional[str]
            """
            Last four characters of the IBAN.
            """
            _inner_class_types = {"generated_from": GeneratedFrom}

        class Shopeepay(StripeObject):
            pass

        class Sofort(StripeObject):
            country: Optional[str]
            """
            Two-letter ISO code representing the country the bank account is located in.
            """

        class StripeBalance(StripeObject):
            account: Optional[str]
            """
            The connected account ID whose Stripe balance to use as the source of payment
            """

        class Sunbit(StripeObject):
            pass

        class Swish(StripeObject):
            pass

        class Tamara(StripeObject):
            pass

        class Twint(StripeObject):
            pass

        class Upi(StripeObject):
            vpa: Optional[str]
            """
            Customer's unique Virtual Payment Address
            """

        class UsBankAccount(StripeObject):
            class Networks(StripeObject):
                preferred: Optional[str]
                """
                The preferred network.
                """
                supported: List[Literal["ach", "us_domestic_wire"]]
                """
                All supported networks.
                """

            class StatusDetails(StripeObject):
                class Blocked(StripeObject):
                    network_code: Optional[
                        Literal[
                            "R02",
                            "R03",
                            "R04",
                            "R05",
                            "R07",
                            "R08",
                            "R10",
                            "R11",
                            "R16",
                            "R20",
                            "R29",
                            "R31",
                        ]
                    ]
                    """
                    The ACH network code that resulted in this block.
                    """
                    reason: Optional[
                        Literal[
                            "bank_account_closed",
                            "bank_account_frozen",
                            "bank_account_invalid_details",
                            "bank_account_restricted",
                            "bank_account_unusable",
                            "debit_not_authorized",
                            "tokenized_account_number_deactivated",
                        ]
                    ]
                    """
                    The reason why this PaymentMethod's fingerprint has been blocked
                    """

                blocked: Optional[Blocked]
                _inner_class_types = {"blocked": Blocked}

            account_holder_type: Optional[Literal["company", "individual"]]
            """
            Account holder type: individual or company.
            """
            account_number: Optional[str]
            """
            Account number of the bank account.
            """
            account_type: Optional[Literal["checking", "savings"]]
            """
            Account type: checkings or savings. Defaults to checking if omitted.
            """
            bank_name: Optional[str]
            """
            The name of the bank.
            """
            financial_connections_account: Optional[str]
            """
            The ID of the Financial Connections Account used to create the payment method.
            """
            fingerprint: Optional[str]
            """
            Uniquely identifies this particular bank account. You can use this attribute to check whether two bank accounts are the same.
            """
            last4: Optional[str]
            """
            Last four digits of the bank account number.
            """
            networks: Optional[Networks]
            """
            Contains information about US bank account networks that can be used.
            """
            routing_number: Optional[str]
            """
            Routing number of the bank account.
            """
            status_details: Optional[StatusDetails]
            """
            Contains information about the future reusability of this PaymentMethod.
            """
            _inner_class_types = {
                "networks": Networks,
                "status_details": StatusDetails,
            }

        class WechatPay(StripeObject):
            pass

        class Zip(StripeObject):
            pass

        acss_debit: Optional[AcssDebit]
        affirm: Optional[Affirm]
        afterpay_clearpay: Optional[AfterpayClearpay]
        alipay: Optional[Alipay]
        alma: Optional[Alma]
        amazon_pay: Optional[AmazonPay]
        au_becs_debit: Optional[AuBecsDebit]
        bacs_debit: Optional[BacsDebit]
        bancontact: Optional[Bancontact]
        billie: Optional[Billie]
        billing_details: Optional[BillingDetails]
        """
        Billing information associated with the PaymentMethod that may be used or required by particular types of payment methods.
        """
        bizum: Optional[Bizum]
        blik: Optional[Blik]
        boleto: Optional[Boleto]
        card: Optional[Card]
        card_present: Optional[CardPresent]
        cashapp: Optional[Cashapp]
        crypto: Optional[Crypto]
        customer_balance: Optional[CustomerBalance]
        eps: Optional[Eps]
        fpx: Optional[Fpx]
        gift_card: Optional[GiftCard]
        giropay: Optional[Giropay]
        gopay: Optional[Gopay]
        grabpay: Optional[Grabpay]
        id_bank_transfer: Optional[IdBankTransfer]
        ideal: Optional[Ideal]
        interac_present: Optional[InteracPresent]
        kakao_pay: Optional[KakaoPay]
        klarna: Optional[Klarna]
        konbini: Optional[Konbini]
        kr_card: Optional[KrCard]
        link: Optional[Link]
        mb_way: Optional[MbWay]
        mobilepay: Optional[Mobilepay]
        multibanco: Optional[Multibanco]
        naver_pay: Optional[NaverPay]
        nz_bank_account: Optional[NzBankAccount]
        oxxo: Optional[Oxxo]
        p24: Optional[P24]
        pay_by_bank: Optional[PayByBank]
        payco: Optional[Payco]
        paynow: Optional[Paynow]
        paypal: Optional[Paypal]
        paypay: Optional[Paypay]
        payto: Optional[Payto]
        pix: Optional[Pix]
        promptpay: Optional[Promptpay]
        qris: Optional[Qris]
        rechnung: Optional[Rechnung]
        revolut_pay: Optional[RevolutPay]
        samsung_pay: Optional[SamsungPay]
        satispay: Optional[Satispay]
        scalapay: Optional[Scalapay]
        sepa_debit: Optional[SepaDebit]
        shopeepay: Optional[Shopeepay]
        sofort: Optional[Sofort]
        stripe_balance: Optional[StripeBalance]
        sunbit: Optional[Sunbit]
        swish: Optional[Swish]
        tamara: Optional[Tamara]
        twint: Optional[Twint]
        type: Literal[
            "acss_debit",
            "affirm",
            "afterpay_clearpay",
            "alipay",
            "alma",
            "amazon_pay",
            "au_becs_debit",
            "bacs_debit",
            "bancontact",
            "billie",
            "bizum",
            "blik",
            "boleto",
            "card",
            "card_present",
            "cashapp",
            "crypto",
            "custom",
            "customer_balance",
            "eps",
            "fpx",
            "gift_card",
            "giropay",
            "gopay",
            "grabpay",
            "id_bank_transfer",
            "ideal",
            "interac_present",
            "kakao_pay",
            "klarna",
            "konbini",
            "kr_card",
            "link",
            "mb_way",
            "mobilepay",
            "multibanco",
            "naver_pay",
            "nz_bank_account",
            "oxxo",
            "p24",
            "pay_by_bank",
            "payco",
            "paynow",
            "paypal",
            "paypay",
            "payto",
            "pix",
            "promptpay",
            "qris",
            "rechnung",
            "revolut_pay",
            "samsung_pay",
            "satispay",
            "scalapay",
            "sepa_debit",
            "shopeepay",
            "sofort",
            "stripe_balance",
            "sunbit",
            "swish",
            "tamara",
            "twint",
            "upi",
            "us_bank_account",
            "wechat_pay",
            "zip",
        ]
        """
        The type of the PaymentMethod. An additional hash is included on the PaymentMethod with a name matching this value. It contains additional information specific to the PaymentMethod type.
        """
        upi: Optional[Upi]
        us_bank_account: Optional[UsBankAccount]
        wechat_pay: Optional[WechatPay]
        zip: Optional[Zip]
        _inner_class_types = {
            "acss_debit": AcssDebit,
            "affirm": Affirm,
            "afterpay_clearpay": AfterpayClearpay,
            "alipay": Alipay,
            "alma": Alma,
            "amazon_pay": AmazonPay,
            "au_becs_debit": AuBecsDebit,
            "bacs_debit": BacsDebit,
            "bancontact": Bancontact,
            "billie": Billie,
            "billing_details": BillingDetails,
            "bizum": Bizum,
            "blik": Blik,
            "boleto": Boleto,
            "card": Card,
            "card_present": CardPresent,
            "cashapp": Cashapp,
            "crypto": Crypto,
            "customer_balance": CustomerBalance,
            "eps": Eps,
            "fpx": Fpx,
            "gift_card": GiftCard,
            "giropay": Giropay,
            "gopay": Gopay,
            "grabpay": Grabpay,
            "id_bank_transfer": IdBankTransfer,
            "ideal": Ideal,
            "interac_present": InteracPresent,
            "kakao_pay": KakaoPay,
            "klarna": Klarna,
            "konbini": Konbini,
            "kr_card": KrCard,
            "link": Link,
            "mb_way": MbWay,
            "mobilepay": Mobilepay,
            "multibanco": Multibanco,
            "naver_pay": NaverPay,
            "nz_bank_account": NzBankAccount,
            "oxxo": Oxxo,
            "p24": P24,
            "pay_by_bank": PayByBank,
            "payco": Payco,
            "paynow": Paynow,
            "paypal": Paypal,
            "paypay": Paypay,
            "payto": Payto,
            "pix": Pix,
            "promptpay": Promptpay,
            "qris": Qris,
            "rechnung": Rechnung,
            "revolut_pay": RevolutPay,
            "samsung_pay": SamsungPay,
            "satispay": Satispay,
            "scalapay": Scalapay,
            "sepa_debit": SepaDebit,
            "shopeepay": Shopeepay,
            "sofort": Sofort,
            "stripe_balance": StripeBalance,
            "sunbit": Sunbit,
            "swish": Swish,
            "tamara": Tamara,
            "twint": Twint,
            "upi": Upi,
            "us_bank_account": UsBankAccount,
            "wechat_pay": WechatPay,
            "zip": Zip,
        }

    class RiskDetails(StripeObject):
        class Insights(StripeObject):
            class Bot(StripeObject):
                recommended_action: str
                """
                Recommended action for this insight.
                """
                score: float
                """
                Risk score for this insight.
                """

            class CardIssuerDecline(StripeObject):
                recommended_action: str
                """
                Recommended action for this insight.
                """
                score: float
                """
                Risk score for this insight.
                """

            class CardTesting(StripeObject):
                recommended_action: str
                """
                Recommended action for this insight.
                """
                score: float
                """
                Risk score for this insight.
                """

            class FraudulentDispute(StripeObject):
                recommended_action: str
                """
                Recommended action for this insight.
                """
                score: int
                """
                Risk score for this insight.
                """

            class StolenCard(StripeObject):
                recommended_action: str
                """
                Recommended action for this insight.
                """
                score: int
                """
                Risk score for this insight.
                """

            bot: Optional[Bot]
            """
            Bot risk insight.
            """
            card_issuer_decline: Optional[CardIssuerDecline]
            """
            Card issuer decline risk insight.
            """
            card_testing: Optional[CardTesting]
            """
            Card testing risk insight.
            """
            fraudulent_dispute: Optional[FraudulentDispute]
            """
            Fraudulent dispute risk insight.
            """
            stolen_card: Optional[StolenCard]
            """
            Stolen card risk insight.
            """
            _inner_class_types = {
                "bot": Bot,
                "card_issuer_decline": CardIssuerDecline,
                "card_testing": CardTesting,
                "fraudulent_dispute": FraudulentDispute,
                "stolen_card": StolenCard,
            }

        insights: Insights
        """
        Risk insights for this token, including scores and recommended actions for each risk type.
        """
        _inner_class_types = {"insights": Insights}

    class UsageDetails(StripeObject):
        class AmountCaptured(StripeObject):
            currency: str
            """
            Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
            """
            value: int
            """
            Integer value of the amount in the smallest currency unit.
            """

        amount_captured: Optional[AmountCaptured]
        """
        The total amount captured using this SharedPaymentToken.
        """
        _inner_class_types = {"amount_captured": AmountCaptured}

    class UsageLimits(StripeObject):
        currency: str
        """
        Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
        """
        expires_at: Optional[int]
        """
        Time at which this SharedPaymentToken expires and can no longer be used to confirm a PaymentIntent.
        """
        max_amount: int
        """
        Max amount that can be captured using this SharedPaymentToken.
        """
        recurring_interval: Optional[Literal["month", "week", "year"]]
        """
        The recurring interval at which the shared payment token's amount usage restrictions reset.
        """

    agent_details: Optional[AgentDetails]
    """
    Details about the agent that issued this SharedPaymentGrantedToken.
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    deactivated_at: Optional[int]
    """
    Time at which this SharedPaymentGrantedToken expires and can no longer be used to confirm a PaymentIntent.
    """
    deactivated_reason: Optional[
        Literal["consumed", "expired", "resolved", "revoked"]
    ]
    """
    The reason why the SharedPaymentGrantedToken has been deactivated.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    object: Literal["shared_payment.granted_token"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    payment_method_details: Optional[PaymentMethodDetails]
    """
    Details of the PaymentMethod that was shared via this token.
    """
    risk_details: Optional[RiskDetails]
    """
    Risk details of the SharedPaymentGrantedToken.
    """
    shared_metadata: Optional[UntypedStripeObject[str]]
    """
    Metadata about the SharedPaymentGrantedToken.
    """
    usage_details: Optional[UsageDetails]
    """
    Some details about how the SharedPaymentGrantedToken has been used already.
    """
    usage_limits: Optional[UsageLimits]
    """
    Limits on how this SharedPaymentGrantedToken can be used.
    """

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["GrantedTokenRetrieveParams"]
    ) -> "GrantedToken":
        """
        Retrieves an existing SharedPaymentGrantedToken object
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["GrantedTokenRetrieveParams"]
    ) -> "GrantedToken":
        """
        Retrieves an existing SharedPaymentGrantedToken object
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    class TestHelpers(APIResourceTestHelpers["GrantedToken"]):
        _resource_cls: Type["GrantedToken"]

        @classmethod
        def create(
            cls, **params: Unpack["GrantedTokenCreateParams"]
        ) -> "GrantedToken":
            """
            Creates a new test SharedPaymentGrantedToken object. This endpoint is only available in test mode and allows sellers to create SharedPaymentGrantedTokens for testing their integration
            """
            return cast(
                "GrantedToken",
                cls._static_request(
                    "post",
                    "/v1/test_helpers/shared_payment/granted_tokens",
                    params=params,
                ),
            )

        @classmethod
        async def create_async(
            cls, **params: Unpack["GrantedTokenCreateParams"]
        ) -> "GrantedToken":
            """
            Creates a new test SharedPaymentGrantedToken object. This endpoint is only available in test mode and allows sellers to create SharedPaymentGrantedTokens for testing their integration
            """
            return cast(
                "GrantedToken",
                await cls._static_request_async(
                    "post",
                    "/v1/test_helpers/shared_payment/granted_tokens",
                    params=params,
                ),
            )

        @classmethod
        def _cls_revoke(
            cls,
            shared_payment_granted_token: str,
            **params: Unpack["GrantedTokenRevokeParams"],
        ) -> "GrantedToken":
            """
            Revokes a test SharedPaymentGrantedToken object. This endpoint is only available in test mode and allows sellers to revoke SharedPaymentGrantedTokens for testing their integration
            """
            return cast(
                "GrantedToken",
                cls._static_request(
                    "post",
                    "/v1/test_helpers/shared_payment/granted_tokens/{shared_payment_granted_token}/revoke".format(
                        shared_payment_granted_token=sanitize_id(
                            shared_payment_granted_token
                        )
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        def revoke(
            shared_payment_granted_token: str,
            **params: Unpack["GrantedTokenRevokeParams"],
        ) -> "GrantedToken":
            """
            Revokes a test SharedPaymentGrantedToken object. This endpoint is only available in test mode and allows sellers to revoke SharedPaymentGrantedTokens for testing their integration
            """
            ...

        @overload
        def revoke(
            self, **params: Unpack["GrantedTokenRevokeParams"]
        ) -> "GrantedToken":
            """
            Revokes a test SharedPaymentGrantedToken object. This endpoint is only available in test mode and allows sellers to revoke SharedPaymentGrantedTokens for testing their integration
            """
            ...

        @class_method_variant("_cls_revoke")
        def revoke(  # pyright: ignore[reportGeneralTypeIssues]
            self, **params: Unpack["GrantedTokenRevokeParams"]
        ) -> "GrantedToken":
            """
            Revokes a test SharedPaymentGrantedToken object. This endpoint is only available in test mode and allows sellers to revoke SharedPaymentGrantedTokens for testing their integration
            """
            return cast(
                "GrantedToken",
                self.resource._request(
                    "post",
                    "/v1/test_helpers/shared_payment/granted_tokens/{shared_payment_granted_token}/revoke".format(
                        shared_payment_granted_token=sanitize_id(
                            self.resource._data.get("id")
                        )
                    ),
                    params=params,
                ),
            )

        @classmethod
        async def _cls_revoke_async(
            cls,
            shared_payment_granted_token: str,
            **params: Unpack["GrantedTokenRevokeParams"],
        ) -> "GrantedToken":
            """
            Revokes a test SharedPaymentGrantedToken object. This endpoint is only available in test mode and allows sellers to revoke SharedPaymentGrantedTokens for testing their integration
            """
            return cast(
                "GrantedToken",
                await cls._static_request_async(
                    "post",
                    "/v1/test_helpers/shared_payment/granted_tokens/{shared_payment_granted_token}/revoke".format(
                        shared_payment_granted_token=sanitize_id(
                            shared_payment_granted_token
                        )
                    ),
                    params=params,
                ),
            )

        @overload
        @staticmethod
        async def revoke_async(
            shared_payment_granted_token: str,
            **params: Unpack["GrantedTokenRevokeParams"],
        ) -> "GrantedToken":
            """
            Revokes a test SharedPaymentGrantedToken object. This endpoint is only available in test mode and allows sellers to revoke SharedPaymentGrantedTokens for testing their integration
            """
            ...

        @overload
        async def revoke_async(
            self, **params: Unpack["GrantedTokenRevokeParams"]
        ) -> "GrantedToken":
            """
            Revokes a test SharedPaymentGrantedToken object. This endpoint is only available in test mode and allows sellers to revoke SharedPaymentGrantedTokens for testing their integration
            """
            ...

        @class_method_variant("_cls_revoke_async")
        async def revoke_async(  # pyright: ignore[reportGeneralTypeIssues]
            self, **params: Unpack["GrantedTokenRevokeParams"]
        ) -> "GrantedToken":
            """
            Revokes a test SharedPaymentGrantedToken object. This endpoint is only available in test mode and allows sellers to revoke SharedPaymentGrantedTokens for testing their integration
            """
            return cast(
                "GrantedToken",
                await self.resource._request_async(
                    "post",
                    "/v1/test_helpers/shared_payment/granted_tokens/{shared_payment_granted_token}/revoke".format(
                        shared_payment_granted_token=sanitize_id(
                            self.resource._data.get("id")
                        )
                    ),
                    params=params,
                ),
            )

    @property
    def test_helpers(self):
        return self.TestHelpers(self)

    _inner_class_types = {
        "agent_details": AgentDetails,
        "payment_method_details": PaymentMethodDetails,
        "risk_details": RiskDetails,
        "usage_details": UsageDetails,
        "usage_limits": UsageLimits,
    }


GrantedToken.TestHelpers._resource_cls = GrantedToken



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import List
from typing_extensions import Literal, NotRequired


class PaymentIntentUpdateCryptoRefundAddressParams(RequestOptions):
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    network: Literal["base", "ethereum", "polygon", "solana", "sui", "tempo"]
    """
    The blockchain network for the refund address.
    """
    refund_address: str
    """
    The wallet address that should receive refunds for deposits on the specified network.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._request_options import RequestOptions
    from stripe.issuing._authorization import Authorization
    from stripe.params.test_helpers.issuing._authorization_capture_params import (
        AuthorizationCaptureParams,
    )
    from stripe.params.test_helpers.issuing._authorization_create_params import (
        AuthorizationCreateParams,
    )
    from stripe.params.test_helpers.issuing._authorization_expire_params import (
        AuthorizationExpireParams,
    )
    from stripe.params.test_helpers.issuing._authorization_finalize_amount_params import (
        AuthorizationFinalizeAmountParams,
    )
    from stripe.params.test_helpers.issuing._authorization_increment_params import (
        AuthorizationIncrementParams,
    )
    from stripe.params.test_helpers.issuing._authorization_respond_params import (
        AuthorizationRespondParams,
    )
    from stripe.params.test_helpers.issuing._authorization_reverse_params import (
        AuthorizationReverseParams,
    )


class AuthorizationService(StripeService):
    def create(
        self,
        params: "AuthorizationCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Authorization":
        """
        Create a test-mode authorization.
        """
        return cast(
            "Authorization",
            self._request(
                "post",
                "/v1/test_helpers/issuing/authorizations",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "AuthorizationCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Authorization":
        """
        Create a test-mode authorization.
        """
        return cast(
            "Authorization",
            await self._request_async(
                "post",
                "/v1/test_helpers/issuing/authorizations",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def capture(
        self,
        authorization: str,
        params: Optional["AuthorizationCaptureParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Authorization":
        """
        Capture a test-mode authorization.
        """
        return cast(
            "Authorization",
            self._request(
                "post",
                "/v1/test_helpers/issuing/authorizations/{authorization}/capture".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def capture_async(
        self,
        authorization: str,
        params: Optional["AuthorizationCaptureParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Authorization":
        """
        Capture a test-mode authorization.
        """
        return cast(
            "Authorization",
            await self._request_async(
                "post",
                "/v1/test_helpers/issuing/authorizations/{authorization}/capture".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def expire(
        self,
        authorization: str,
        params: Optional["AuthorizationExpireParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Authorization":
        """
        Expire a test-mode Authorization.
        """
        return cast(
            "Authorization",
            self._request(
                "post",
                "/v1/test_helpers/issuing/authorizations/{authorization}/expire".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def expire_async(
        self,
        authorization: str,
        params: Optional["AuthorizationExpireParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Authorization":
        """
        Expire a test-mode Authorization.
        """
        return cast(
            "Authorization",
            await self._request_async(
                "post",
                "/v1/test_helpers/issuing/authorizations/{authorization}/expire".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def finalize_amount(
        self,
        authorization: str,
        params: "AuthorizationFinalizeAmountParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Authorization":
        """
        Finalize the amount on an Authorization prior to capture, when the initial authorization was for an estimated amount.
        """
        return cast(
            "Authorization",
            self._request(
                "post",
                "/v1/test_helpers/issuing/authorizations/{authorization}/finalize_amount".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def finalize_amount_async(
        self,
        authorization: str,
        params: "AuthorizationFinalizeAmountParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Authorization":
        """
        Finalize the amount on an Authorization prior to capture, when the initial authorization was for an estimated amount.
        """
        return cast(
            "Authorization",
            await self._request_async(
                "post",
                "/v1/test_helpers/issuing/authorizations/{authorization}/finalize_amount".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def respond(
        self,
        authorization: str,
        params: "AuthorizationRespondParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Authorization":
        """
        Respond to a fraud challenge on a testmode Issuing authorization, simulating either a confirmation of fraud or a correction of legitimacy.
        """
        return cast(
            "Authorization",
            self._request(
                "post",
                "/v1/test_helpers/issuing/authorizations/{authorization}/fraud_challenges/respond".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def respond_async(
        self,
        authorization: str,
        params: "AuthorizationRespondParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Authorization":
        """
        Respond to a fraud challenge on a testmode Issuing authorization, simulating either a confirmation of fraud or a correction of legitimacy.
        """
        return cast(
            "Authorization",
            await self._request_async(
                "post",
                "/v1/test_helpers/issuing/authorizations/{authorization}/fraud_challenges/respond".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def increment(
        self,
        authorization: str,
        params: "AuthorizationIncrementParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Authorization":
        """
        Increment a test-mode Authorization.
        """
        return cast(
            "Authorization",
            self._request(
                "post",
                "/v1/test_helpers/issuing/authorizations/{authorization}/increment".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def increment_async(
        self,
        authorization: str,
        params: "AuthorizationIncrementParams",
        options: Optional["RequestOptions"] = None,
    ) -> "Authorization":
        """
        Increment a test-mode Authorization.
        """
        return cast(
            "Authorization",
            await self._request_async(
                "post",
                "/v1/test_helpers/issuing/authorizations/{authorization}/increment".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def reverse(
        self,
        authorization: str,
        params: Optional["AuthorizationReverseParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Authorization":
        """
        Reverse a test-mode Authorization.
        """
        return cast(
            "Authorization",
            self._request(
                "post",
                "/v1/test_helpers/issuing/authorizations/{authorization}/reverse".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def reverse_async(
        self,
        authorization: str,
        params: Optional["AuthorizationReverseParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "Authorization":
        """
        Reverse a test-mode Authorization.
        """
        return cast(
            "Authorization",
            await self._request_async(
                "post",
                "/v1/test_helpers/issuing/authorizations/{authorization}/reverse".format(
                    authorization=sanitize_id(authorization),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._expandable_field import ExpandableField
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import StripeObject
from typing import ClassVar, List, Optional, Union
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._application_fee import ApplicationFee
    from stripe._application_fee_refund import ApplicationFeeRefund
    from stripe._charge import Charge
    from stripe._connect_collection_transfer import ConnectCollectionTransfer
    from stripe._customer_cash_balance_transaction import (
        CustomerCashBalanceTransaction,
    )
    from stripe._dispute import Dispute as DisputeResource
    from stripe._payout import Payout
    from stripe._refund import Refund
    from stripe._reserve_transaction import ReserveTransaction
    from stripe._reversal import Reversal
    from stripe._tax_deducted_at_source import TaxDeductedAtSource
    from stripe._topup import Topup
    from stripe._transfer import Transfer
    from stripe.issuing._authorization import Authorization
    from stripe.issuing._dispute import Dispute as IssuingDisputeResource
    from stripe.issuing._transaction import Transaction
    from stripe.params._balance_transaction_list_params import (
        BalanceTransactionListParams,
    )
    from stripe.params._balance_transaction_retrieve_params import (
        BalanceTransactionRetrieveParams,
    )


class BalanceTransaction(ListableAPIResource["BalanceTransaction"]):
    """
    Balance transactions represent funds moving through your Stripe account.
    Stripe creates them for every type of transaction that enters or leaves your Stripe account balance.

    Related guide: [Balance transaction types](https://docs.stripe.com/reports/balance-transaction-types)
    """

    OBJECT_NAME: ClassVar[Literal["balance_transaction"]] = (
        "balance_transaction"
    )

    class FeeDetail(StripeObject):
        amount: int
        """
        Amount of the fee, in cents.
        """
        application: Optional[str]
        """
        ID of the Connect application that earned the fee.
        """
        currency: str
        """
        Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
        """
        description: Optional[str]
        """
        An arbitrary string attached to the object. Often useful for displaying to users.
        """
        type: str
        """
        Type of the fee, one of: `application_fee`, `payment_method_passthrough_fee`, `stripe_fee`, `tax`, or `withheld_tax`.
        """

    amount: int
    """
    Gross amount of this transaction (in cents (or local equivalent)). A positive value represents funds charged to another party, and a negative value represents funds sent to another party.
    """
    available_on: int
    """
    The date that the transaction's net funds become available in the Stripe balance.
    """
    balance_type: Literal[
        "fee_credits",
        "issuing",
        "payments",
        "refund_and_dispute_prefunding",
        "risk_reserved",
        "transit",
    ]
    """
    The balance that this transaction impacts.
    """
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    currency: str
    """
    Three-letter [ISO currency code](https://www.iso.org/iso-4217-currency-codes.html), in lowercase. Must be a [supported currency](https://stripe.com/docs/currencies).
    """
    description: Optional[str]
    """
    An arbitrary string attached to the object. Often useful for displaying to users.
    """
    exchange_rate: Optional[float]
    """
    If applicable, this transaction uses an exchange rate. If money converts from currency A to currency B, then the `amount` in currency A, multipled by the `exchange_rate`, equals the `amount` in currency B. For example, if you charge a customer 10.00 EUR, the PaymentIntent's `amount` is `1000` and `currency` is `eur`. If this converts to 12.34 USD in your Stripe account, the BalanceTransaction's `amount` is `1234`, its `currency` is `usd`, and the `exchange_rate` is `1.234`.
    """
    fee: int
    """
    Fees (in cents (or local equivalent)) paid for this transaction. Represented as a positive integer when assessed.
    """
    fee_details: List[FeeDetail]
    """
    Detailed breakdown of fees (in cents (or local equivalent)) paid for this transaction.
    """
    id: str
    """
    Unique identifier for the object.
    """
    net: int
    """
    Net impact to a Stripe balance (in cents (or local equivalent)). A positive value represents incrementing a Stripe balance, and a negative value decrementing a Stripe balance. You can calculate the net impact of a transaction on a balance by `amount` - `fee`
    """
    object: Literal["balance_transaction"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    reporting_category: str
    """
    Learn more about how [reporting categories](https://stripe.com/docs/reports/reporting-categories) can help you understand balance transactions from an accounting perspective.
    """
    source: Optional[
        ExpandableField[
            Union[
                "ApplicationFee",
                "Charge",
                "ConnectCollectionTransfer",
                "CustomerCashBalanceTransaction",
                "DisputeResource",
                "ApplicationFeeRefund",
                "Authorization",
                "IssuingDisputeResource",
                "Transaction",
                "Payout",
                "Refund",
                "ReserveTransaction",
                "TaxDeductedAtSource",
                "Topup",
                "Transfer",
                "Reversal",
            ]
        ]
    ]
    """
    This transaction relates to the Stripe object.
    """
    status: str
    """
    The transaction's net funds status in the Stripe balance, which are either `available` or `pending`.
    """
    type: Literal[
        "adjustment",
        "advance",
        "advance_funding",
        "anticipation_repayment",
        "application_fee",
        "application_fee_refund",
        "charge",
        "climate_order_purchase",
        "climate_order_refund",
        "connect_collection_transfer",
        "contribution",
        "fee_credit_funding",
        "inbound_transfer",
        "inbound_transfer_reversal",
        "issuing_authorization_hold",
        "issuing_authorization_release",
        "issuing_dispute",
        "issuing_transaction",
        "obligation_outbound",
        "obligation_reversal_inbound",
        "payment",
        "payment_failure_refund",
        "payment_network_reserve_hold",
        "payment_network_reserve_release",
        "payment_refund",
        "payment_reversal",
        "payment_unreconciled",
        "payout",
        "payout_cancel",
        "payout_failure",
        "payout_minimum_balance_hold",
        "payout_minimum_balance_release",
        "refund",
        "refund_failure",
        "reserve_hold",
        "reserve_release",
        "reserve_transaction",
        "reserved_funds",
        "stripe_balance_payment_debit",
        "stripe_balance_payment_debit_reversal",
        "stripe_fee",
        "stripe_fx_fee",
        "tax_fee",
        "tax_fund",
        "topup",
        "topup_reversal",
        "transfer",
        "transfer_cancel",
        "transfer_failure",
        "transfer_refund",
    ]
    """
    Transaction type: `tax_fund`, `adjustment`, `advance`, `advance_funding`, `anticipation_repayment`, `application_fee`, `application_fee_refund`, `charge`, `climate_order_purchase`, `climate_order_refund`, `connect_collection_transfer`, `contribution`, `inbound_transfer`, `inbound_transfer_reversal`, `issuing_authorization_hold`, `issuing_authorization_release`, `issuing_dispute`, `issuing_transaction`, `obligation_outbound`, `obligation_reversal_inbound`, `payment`, `payment_failure_refund`, `payment_network_reserve_hold`, `payment_network_reserve_release`, `payment_refund`, `payment_reversal`, `payment_unreconciled`, `payout`, `payout_cancel`, `payout_failure`, `payout_minimum_balance_hold`, `payout_minimum_balance_release`, `refund`, `refund_failure`, `reserve_transaction`, `reserved_funds`, `reserve_hold`, `reserve_release`, `stripe_fee`, `stripe_fx_fee`, `stripe_balance_payment_debit`, `stripe_balance_payment_debit_reversal`, `tax_fee`, `topup`, `topup_reversal`, `transfer`, `transfer_cancel`, `transfer_failure`, `transfer_refund`, or `fee_credit_funding`. Learn more about [balance transaction types and what they represent](https://stripe.com/docs/reports/balance-transaction-types). To classify transactions for accounting purposes, consider `reporting_category` instead.
    """

    @classmethod
    def list(
        cls, **params: Unpack["BalanceTransactionListParams"]
    ) -> ListObject["BalanceTransaction"]:
        """
        Returns a list of transactions that have contributed to the Stripe account balance (for example, charges, transfers, and so on). The transactions return in sorted order, with the most recent transactions appearing first.

        The previous name of this endpoint was “Balance history,” and it used the path /v1/balance/history.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["BalanceTransactionListParams"]
    ) -> ListObject["BalanceTransaction"]:
        """
        Returns a list of transactions that have contributed to the Stripe account balance (for example, charges, transfers, and so on). The transactions return in sorted order, with the most recent transactions appearing first.

        The previous name of this endpoint was “Balance history,” and it used the path /v1/balance/history.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def retrieve(
        cls, id: str, **params: Unpack["BalanceTransactionRetrieveParams"]
    ) -> "BalanceTransaction":
        """
        Retrieves the balance transaction with the given ID.

        Note that this endpoint previously used the path /v1/balance/history/:id.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls, id: str, **params: Unpack["BalanceTransactionRetrieveParams"]
    ) -> "BalanceTransaction":
        """
        Retrieves the balance transaction with the given ID.

        Note that this endpoint previously used the path /v1/balance/history/:id.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    _inner_class_types = {"fee_details": FeeDetail}



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._list_object import ListObject
from stripe._listable_api_resource import ListableAPIResource
from stripe._stripe_object import UntypedStripeObject
from stripe._util import class_method_variant, sanitize_id
from typing import ClassVar, Optional, cast, overload
from typing_extensions import Literal, Unpack, TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params._invoice_rendering_template_archive_params import (
        InvoiceRenderingTemplateArchiveParams,
    )
    from stripe.params._invoice_rendering_template_list_params import (
        InvoiceRenderingTemplateListParams,
    )
    from stripe.params._invoice_rendering_template_retrieve_params import (
        InvoiceRenderingTemplateRetrieveParams,
    )
    from stripe.params._invoice_rendering_template_unarchive_params import (
        InvoiceRenderingTemplateUnarchiveParams,
    )


class InvoiceRenderingTemplate(
    ListableAPIResource["InvoiceRenderingTemplate"]
):
    """
    Invoice Rendering Templates are used to configure how invoices are rendered on surfaces like the PDF. Invoice Rendering Templates
    can be created from within the Dashboard, and they can be used over the API when creating invoices.
    """

    OBJECT_NAME: ClassVar[Literal["invoice_rendering_template"]] = (
        "invoice_rendering_template"
    )
    created: int
    """
    Time at which the object was created. Measured in seconds since the Unix epoch.
    """
    id: str
    """
    Unique identifier for the object.
    """
    livemode: bool
    """
    If the object exists in live mode, the value is `true`. If the object exists in test mode, the value is `false`.
    """
    metadata: Optional[UntypedStripeObject[str]]
    """
    Set of [key-value pairs](https://docs.stripe.com/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format.
    """
    nickname: Optional[str]
    """
    A brief description of the template, hidden from customers
    """
    object: Literal["invoice_rendering_template"]
    """
    String representing the object's type. Objects of the same type share the same value.
    """
    status: Literal["active", "archived"]
    """
    The status of the template, one of `active` or `archived`.
    """
    version: int
    """
    Version of this template; version increases by one when an update on the template changes any field that controls invoice rendering
    """

    @classmethod
    def _cls_archive(
        cls,
        template: str,
        **params: Unpack["InvoiceRenderingTemplateArchiveParams"],
    ) -> "InvoiceRenderingTemplate":
        """
        Updates the status of an invoice rendering template to ‘archived' so no new Stripe objects (customers, invoices, etc.) can reference it. The template can also no longer be updated. However, if the template is already set on a Stripe object, it will continue to be applied on invoices generated by it.
        """
        return cast(
            "InvoiceRenderingTemplate",
            cls._static_request(
                "post",
                "/v1/invoice_rendering_templates/{template}/archive".format(
                    template=sanitize_id(template)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def archive(
        template: str,
        **params: Unpack["InvoiceRenderingTemplateArchiveParams"],
    ) -> "InvoiceRenderingTemplate":
        """
        Updates the status of an invoice rendering template to ‘archived' so no new Stripe objects (customers, invoices, etc.) can reference it. The template can also no longer be updated. However, if the template is already set on a Stripe object, it will continue to be applied on invoices generated by it.
        """
        ...

    @overload
    def archive(
        self, **params: Unpack["InvoiceRenderingTemplateArchiveParams"]
    ) -> "InvoiceRenderingTemplate":
        """
        Updates the status of an invoice rendering template to ‘archived' so no new Stripe objects (customers, invoices, etc.) can reference it. The template can also no longer be updated. However, if the template is already set on a Stripe object, it will continue to be applied on invoices generated by it.
        """
        ...

    @class_method_variant("_cls_archive")
    def archive(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["InvoiceRenderingTemplateArchiveParams"]
    ) -> "InvoiceRenderingTemplate":
        """
        Updates the status of an invoice rendering template to ‘archived' so no new Stripe objects (customers, invoices, etc.) can reference it. The template can also no longer be updated. However, if the template is already set on a Stripe object, it will continue to be applied on invoices generated by it.
        """
        return cast(
            "InvoiceRenderingTemplate",
            self._request(
                "post",
                "/v1/invoice_rendering_templates/{template}/archive".format(
                    template=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_archive_async(
        cls,
        template: str,
        **params: Unpack["InvoiceRenderingTemplateArchiveParams"],
    ) -> "InvoiceRenderingTemplate":
        """
        Updates the status of an invoice rendering template to ‘archived' so no new Stripe objects (customers, invoices, etc.) can reference it. The template can also no longer be updated. However, if the template is already set on a Stripe object, it will continue to be applied on invoices generated by it.
        """
        return cast(
            "InvoiceRenderingTemplate",
            await cls._static_request_async(
                "post",
                "/v1/invoice_rendering_templates/{template}/archive".format(
                    template=sanitize_id(template)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def archive_async(
        template: str,
        **params: Unpack["InvoiceRenderingTemplateArchiveParams"],
    ) -> "InvoiceRenderingTemplate":
        """
        Updates the status of an invoice rendering template to ‘archived' so no new Stripe objects (customers, invoices, etc.) can reference it. The template can also no longer be updated. However, if the template is already set on a Stripe object, it will continue to be applied on invoices generated by it.
        """
        ...

    @overload
    async def archive_async(
        self, **params: Unpack["InvoiceRenderingTemplateArchiveParams"]
    ) -> "InvoiceRenderingTemplate":
        """
        Updates the status of an invoice rendering template to ‘archived' so no new Stripe objects (customers, invoices, etc.) can reference it. The template can also no longer be updated. However, if the template is already set on a Stripe object, it will continue to be applied on invoices generated by it.
        """
        ...

    @class_method_variant("_cls_archive_async")
    async def archive_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["InvoiceRenderingTemplateArchiveParams"]
    ) -> "InvoiceRenderingTemplate":
        """
        Updates the status of an invoice rendering template to ‘archived' so no new Stripe objects (customers, invoices, etc.) can reference it. The template can also no longer be updated. However, if the template is already set on a Stripe object, it will continue to be applied on invoices generated by it.
        """
        return cast(
            "InvoiceRenderingTemplate",
            await self._request_async(
                "post",
                "/v1/invoice_rendering_templates/{template}/archive".format(
                    template=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    def list(
        cls, **params: Unpack["InvoiceRenderingTemplateListParams"]
    ) -> ListObject["InvoiceRenderingTemplate"]:
        """
        List all templates, ordered by creation date, with the most recently created template appearing first.
        """
        result = cls._static_request(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    async def list_async(
        cls, **params: Unpack["InvoiceRenderingTemplateListParams"]
    ) -> ListObject["InvoiceRenderingTemplate"]:
        """
        List all templates, ordered by creation date, with the most recently created template appearing first.
        """
        result = await cls._static_request_async(
            "get",
            cls.class_url(),
            params=params,
        )
        if not isinstance(result, ListObject):
            raise TypeError(
                "Expected list object from API, got %s"
                % (type(result).__name__)
            )

        return result

    @classmethod
    def retrieve(
        cls,
        id: str,
        **params: Unpack["InvoiceRenderingTemplateRetrieveParams"],
    ) -> "InvoiceRenderingTemplate":
        """
        Retrieves an invoice rendering template with the given ID. It by default returns the latest version of the template. Optionally, specify a version to see previous versions.
        """
        instance = cls(id, **params)
        instance.refresh()
        return instance

    @classmethod
    async def retrieve_async(
        cls,
        id: str,
        **params: Unpack["InvoiceRenderingTemplateRetrieveParams"],
    ) -> "InvoiceRenderingTemplate":
        """
        Retrieves an invoice rendering template with the given ID. It by default returns the latest version of the template. Optionally, specify a version to see previous versions.
        """
        instance = cls(id, **params)
        await instance.refresh_async()
        return instance

    @classmethod
    def _cls_unarchive(
        cls,
        template: str,
        **params: Unpack["InvoiceRenderingTemplateUnarchiveParams"],
    ) -> "InvoiceRenderingTemplate":
        """
        Unarchive an invoice rendering template so it can be used on new Stripe objects again.
        """
        return cast(
            "InvoiceRenderingTemplate",
            cls._static_request(
                "post",
                "/v1/invoice_rendering_templates/{template}/unarchive".format(
                    template=sanitize_id(template)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    def unarchive(
        template: str,
        **params: Unpack["InvoiceRenderingTemplateUnarchiveParams"],
    ) -> "InvoiceRenderingTemplate":
        """
        Unarchive an invoice rendering template so it can be used on new Stripe objects again.
        """
        ...

    @overload
    def unarchive(
        self, **params: Unpack["InvoiceRenderingTemplateUnarchiveParams"]
    ) -> "InvoiceRenderingTemplate":
        """
        Unarchive an invoice rendering template so it can be used on new Stripe objects again.
        """
        ...

    @class_method_variant("_cls_unarchive")
    def unarchive(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["InvoiceRenderingTemplateUnarchiveParams"]
    ) -> "InvoiceRenderingTemplate":
        """
        Unarchive an invoice rendering template so it can be used on new Stripe objects again.
        """
        return cast(
            "InvoiceRenderingTemplate",
            self._request(
                "post",
                "/v1/invoice_rendering_templates/{template}/unarchive".format(
                    template=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )

    @classmethod
    async def _cls_unarchive_async(
        cls,
        template: str,
        **params: Unpack["InvoiceRenderingTemplateUnarchiveParams"],
    ) -> "InvoiceRenderingTemplate":
        """
        Unarchive an invoice rendering template so it can be used on new Stripe objects again.
        """
        return cast(
            "InvoiceRenderingTemplate",
            await cls._static_request_async(
                "post",
                "/v1/invoice_rendering_templates/{template}/unarchive".format(
                    template=sanitize_id(template)
                ),
                params=params,
            ),
        )

    @overload
    @staticmethod
    async def unarchive_async(
        template: str,
        **params: Unpack["InvoiceRenderingTemplateUnarchiveParams"],
    ) -> "InvoiceRenderingTemplate":
        """
        Unarchive an invoice rendering template so it can be used on new Stripe objects again.
        """
        ...

    @overload
    async def unarchive_async(
        self, **params: Unpack["InvoiceRenderingTemplateUnarchiveParams"]
    ) -> "InvoiceRenderingTemplate":
        """
        Unarchive an invoice rendering template so it can be used on new Stripe objects again.
        """
        ...

    @class_method_variant("_cls_unarchive_async")
    async def unarchive_async(  # pyright: ignore[reportGeneralTypeIssues]
        self, **params: Unpack["InvoiceRenderingTemplateUnarchiveParams"]
    ) -> "InvoiceRenderingTemplate":
        """
        Unarchive an invoice rendering template so it can be used on new Stripe objects again.
        """
        return cast(
            "InvoiceRenderingTemplate",
            await self._request_async(
                "post",
                "/v1/invoice_rendering_templates/{template}/unarchive".format(
                    template=sanitize_id(self._data.get("id"))
                ),
                params=params,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe._fr_meal_vouchers_onboarding import FrMealVouchersOnboarding
    from stripe._list_object import ListObject
    from stripe._request_options import RequestOptions
    from stripe.params._fr_meal_vouchers_onboarding_create_params import (
        FrMealVouchersOnboardingCreateParams,
    )
    from stripe.params._fr_meal_vouchers_onboarding_list_params import (
        FrMealVouchersOnboardingListParams,
    )
    from stripe.params._fr_meal_vouchers_onboarding_retrieve_params import (
        FrMealVouchersOnboardingRetrieveParams,
    )
    from stripe.params._fr_meal_vouchers_onboarding_update_params import (
        FrMealVouchersOnboardingUpdateParams,
    )


class FrMealVouchersOnboardingService(StripeService):
    def list(
        self,
        params: Optional["FrMealVouchersOnboardingListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[FrMealVouchersOnboarding]":
        """
        Lists French Meal Vouchers Onboarding objects. The objects are returned in sorted order, with the most recently created objects appearing first.
        """
        return cast(
            "ListObject[FrMealVouchersOnboarding]",
            self._request(
                "get",
                "/v1/fr_meal_vouchers_onboardings",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["FrMealVouchersOnboardingListParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "ListObject[FrMealVouchersOnboarding]":
        """
        Lists French Meal Vouchers Onboarding objects. The objects are returned in sorted order, with the most recently created objects appearing first.
        """
        return cast(
            "ListObject[FrMealVouchersOnboarding]",
            await self._request_async(
                "get",
                "/v1/fr_meal_vouchers_onboardings",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "FrMealVouchersOnboardingCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "FrMealVouchersOnboarding":
        """
        Creates a French Meal Vouchers Onboarding object that represents a restaurant's onboarding status and starts the onboarding process.
        """
        return cast(
            "FrMealVouchersOnboarding",
            self._request(
                "post",
                "/v1/fr_meal_vouchers_onboardings",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "FrMealVouchersOnboardingCreateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "FrMealVouchersOnboarding":
        """
        Creates a French Meal Vouchers Onboarding object that represents a restaurant's onboarding status and starts the onboarding process.
        """
        return cast(
            "FrMealVouchersOnboarding",
            await self._request_async(
                "post",
                "/v1/fr_meal_vouchers_onboardings",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["FrMealVouchersOnboardingRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "FrMealVouchersOnboarding":
        """
        Retrieves the details of a previously created French Meal Vouchers Onboarding object.

        Supply the unique French Meal Vouchers Onboarding ID that was returned from your previous request,
        and Stripe returns the corresponding onboarding information.
        """
        return cast(
            "FrMealVouchersOnboarding",
            self._request(
                "get",
                "/v1/fr_meal_vouchers_onboardings/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["FrMealVouchersOnboardingRetrieveParams"] = None,
        options: Optional["RequestOptions"] = None,
    ) -> "FrMealVouchersOnboarding":
        """
        Retrieves the details of a previously created French Meal Vouchers Onboarding object.

        Supply the unique French Meal Vouchers Onboarding ID that was returned from your previous request,
        and Stripe returns the corresponding onboarding information.
        """
        return cast(
            "FrMealVouchersOnboarding",
            await self._request_async(
                "get",
                "/v1/fr_meal_vouchers_onboardings/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        id: str,
        params: "FrMealVouchersOnboardingUpdateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "FrMealVouchersOnboarding":
        """
        Updates the details of a restaurant's French Meal Vouchers Onboarding object by
        setting the values of the parameters passed. Any parameters not provided are left unchanged.
        After you update the object, the onboarding process automatically restarts.

        You can only update French Meal Vouchers Onboarding objects with the postal_code field requirement in past_due.
        """
        return cast(
            "FrMealVouchersOnboarding",
            self._request(
                "post",
                "/v1/fr_meal_vouchers_onboardings/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        id: str,
        params: "FrMealVouchersOnboardingUpdateParams",
        options: Optional["RequestOptions"] = None,
    ) -> "FrMealVouchersOnboarding":
        """
        Updates the details of a restaurant's French Meal Vouchers Onboarding object by
        setting the values of the parameters passed. Any parameters not provided are left unchanged.
        After you update the object, the onboarding process automatically restarts.

        You can only update French Meal Vouchers Onboarding objects with the postal_code field requirement in past_due.
        """
        return cast(
            "FrMealVouchersOnboarding",
            await self._request_async(
                "post",
                "/v1/fr_meal_vouchers_onboardings/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )
